package es.alten.diputacion.add.modelo.struts;

import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;

import es.alten.diputacion.add.abstraccion.ObjetoDominio;

public abstract class ActionBaseForm extends ActionForm{
	
	private static final long serialVersionUID = 1L;
	private static final Logger LOGGER = Logger.getLogger(ActionBaseForm.class);
	private static final SimpleDateFormat SDF = new SimpleDateFormat("dd/MM/yyyy");
	private static final SimpleDateFormat SDTF = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	/** nombre del bean que realizara la accion. */
	private String accion;

	/**
	 * @return the accion
	 */
	public final String getAccion() {
		return accion;
	}

	/**
	 * @param acc
	 *            the accion to set
	 */
	public void setOperacion(final String acc) {
		this.accion = acc;
	}

	/**
	 * 
	 * @return devuelve el objeto dominio que almacena los datos del formulario.
	 */
	public abstract ObjetoDominio getBean();

	/**
	 * Convierte un String a Long, o devuelve null si falla
	 * @param valor Numero a convertir de String a Long
	 * @return Valor convertido a Long o null si el formato no es correcto
	 */
	public Long getAsLong(String valor) {
		try {
			return Long.valueOf(valor);
		} catch (NumberFormatException e) {
			LOGGER.debug("Ha fallado al parsear como Long el valor " + valor);
			return null;
		}
	}

	/**
	 * Convierte un String a Integer, o devuelve null si falla
	 * @param valor Numero a convertir de String a Integer
	 * @return Valor convertido a Integer o null si el formato no es correcto
	 */
	public Integer getAsInteger(String valor) {
		try {
			return Integer.valueOf(valor);
		} catch (NumberFormatException e) {
			LOGGER.debug("Ha fallado al parsear como Integer el valor " + valor);
			return null;
		}
	}

	/**
	 * Convierte un String a Date, o devuelve null si falla
	 * @param valor Fecha a convertir de String a Date
	 * @return Valor convertido a Date o null si el formato no es correcto
	 */
	public Date getAsDate(String valor) {
		try {
			return SDF.parse(valor);
		} catch (ParseException e) {
			LOGGER.debug("Ha fallado al parsear como Date el valor " + valor);
			return null;
		}
	}

	/**
	 * Convierte un String a Date, o devuelve null si falla
	 * @param valor Fecha a convertir de String a Date
	 * @return Valor convertido a Date o null si el formato no es correcto
	 */
	public Date getAsDateTimeFormat(String valor) {
		try {
			return SDTF.parse(valor);
		} catch (ParseException e) {
			LOGGER.debug("Ha fallado al parsear como Date el valor " + valor);
			return null;
		}
	}
	
	/**
	 * Convierte un String a Float, o devuelve null si falla
	 * @param valor Numero a convertir de String a Float
	 * @return Valor convertido a Float o null si el formato no es correcto
	 */
	public Float getAsFloat(String valor) {
		try {
			valor = valor.replaceAll(",", ".");
			return Float.valueOf(valor);
		} catch (NumberFormatException e) {
			LOGGER.debug("Ha fallado al parsear como Float el valor " + valor);
			return null;
		}
	}
	
	/**
	 * Convierte un String a Double, o devuelve null si falla
	 * @param valor Numero a convertir de String a Double
	 * @return Valor convertido a Double o null si el formato no es correcto
	 */
	public Double getAsDouble(String valor) {
		try {
			return Double.valueOf(valor);
		} catch (NumberFormatException e) {
			LOGGER.debug("Ha fallado al parsear como Double el valor " + valor);
			return null;
		}
	}
	
	public BigDecimal getAsBigDecimal(String valor) {
		try {
			return new BigDecimal(valor);//.valueOf(getAsLong(valor));
		} catch (NumberFormatException e) {
			LOGGER.debug("Ha fallado al parsear como Big Decinmal el valor " + valor);
			return null;
		}
	}
	

	/**
	 * Convierte el resultado de un checkbox en un Boolean
	 * @param valor Valor del checkbox (sera siempre "on" o no se llamaria)
	 * @return Valor convertido a Boolean
	 */
	public Boolean getAsBoolean(String valor) {
		return "on".equalsIgnoreCase(valor);
	}
	
	protected Date isFecha(String _cadenaFechaAValidar) {
		SimpleDateFormat formato = new SimpleDateFormat("dd/MM/yyyy");
		Date fecha = null;
		// Comprobar el formato dd/mm/yyyy
		if (!((_cadenaFechaAValidar.equals("")) || (_cadenaFechaAValidar == null))) {
			try {
				fecha = formato.parse(_cadenaFechaAValidar);
			} catch (ParseException ex) {
				throw new es.alten.diputacion.add.excepcion.CronosWebException();
			}
		}
		// Eliminar los espacios en blanco de la cadena
		_cadenaFechaAValidar = _cadenaFechaAValidar.trim();
		// Dividir la cadena por tokens
		StringTokenizer cadenaFormato = new StringTokenizer(
				_cadenaFechaAValidar, "/");
		// Array que contendr el da, mes y ao de la cadena trozeada
		int[] campos = new int[3];
		int numeroCampo = 0;
		// Aadir al array de enteros el da, mes y ao
		while (cadenaFormato.hasMoreTokens()) {
			campos[numeroCampo] = Integer.parseInt(cadenaFormato.nextToken());
			numeroCampo++;
		}
		// Validar el array de enteros
		isFechaValida(campos);
		// Devolver fecha validada
		return fecha;
	}

	/**
	 * Validacin del rango de da, mes y ao correspondiente a una fecha
	 * concreta.
	 * 
	 * @param cadenaNumeros
	 *            Array que contiene el dia, mes y ao de una fecha concreta.
	 */
	
	private void isFechaValida(int[] _cadenaNumeros) {
		Calendar fechaActual = Calendar.getInstance();
		if (((_cadenaNumeros[0] < 1) || (_cadenaNumeros[0] > 31))
				|| ((_cadenaNumeros[1] < 1) || (_cadenaNumeros[1] > 12))
				|| ((_cadenaNumeros[2] < 1950) || (_cadenaNumeros[2] > (fechaActual
						.get(Calendar.YEAR) + 100)))) {
			throw new es.alten.diputacion.add.excepcion.CronosWebException();
		}
	}
	
	public String fechaFormateada (Date fecha){
		
		DateFormat formato = new SimpleDateFormat("dd/MM/yyyy");   
        String fechaStr = formato.format(fecha);
        return fechaStr;

	}
}
