package es.alten.diputacion.add.servicios.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import es.alten.diputacion.add.abstraccion.AsentamientoDao;
import es.alten.diputacion.add.abstraccion.PersonaDao;
import es.alten.diputacion.add.abstraccion.UOrganizativaDao;
import es.alten.diputacion.add.excepcion.CronosAbstraccionException;
import es.alten.diputacion.add.excepcion.CronosValidacionException;
import es.alten.diputacion.add.modelo.Asentamiento;
import es.alten.diputacion.add.modelo.Persona;
import es.alten.diputacion.add.modelo.UOrganizativa;
import es.alten.diputacion.add.modelo.Usuario;
import es.alten.diputacion.add.modelo.resultado.AsentamientoResultado;
import es.alten.diputacion.add.modelo.resultado.AsentamientoResultado.ConfirmacionAsentamiento;
import es.alten.diputacion.add.modelo.struts.AsentamientoForm;
import es.alten.diputacion.add.modelo.struts.DocumentoPendienteForm;
import es.alten.diputacion.add.servicios.AsentamientoServicio;
import es.alten.diputacion.add.util.CSVAddUtils;
import es.alten.diputacion.add.util.TipoCondicion;

public class AsentamientoServicioImpl implements AsentamientoServicio {

	private AsentamientoDao daoAsentamiento;
	private UOrganizativaDao unidadDAO;
	private PersonaDao personaDAO;
	
/*
	@Override
	public List<Asentamiento> listaAsentamientosPorServicio(
			String codUnidadOrganica) {
		return daoAsentamiento
				.consultaAsentamientoPorUnidadOrganica(codUnidadOrganica);
	}
	
	*/

	public AsentamientoDao getDaoAsentamiento() {
		return daoAsentamiento;
	}

	public void setDaoAsentamiento(AsentamientoDao daoAsentamiento) {
		this.daoAsentamiento = daoAsentamiento;
	}

	public UOrganizativaDao getUnidadDAO() {
		return unidadDAO;
	}

	public void setUnidadDAO(UOrganizativaDao unidadDAO) {
		this.unidadDAO = unidadDAO;
	}
	
	public PersonaDao getPersonaDAO() {
		return personaDAO;
	}

	public void setPersonaDAO(PersonaDao personaDAO) {
		this.personaDAO = personaDAO;
	}

	//Obtiene las unidades y asentamientos pendientes de un usuario
	public List<UOrganizativa> obtenerAsentamientosPendientes(Usuario usuario) {

		List<UOrganizativa> unidades = unidadDAO
				.consultarUOrganizativaUsuario(usuario.getCodigo());

		for (UOrganizativa unidad : unidades) {

			List<Asentamiento> asentamientos = daoAsentamiento.consultaAsentamientoPorUnidadOrganica(unidad.getCodUOrganizativa());
			unidad.setAsentamientos(asentamientos);
		}

		return unidades;

	}
	
	//Busca Asentamientos
	public AsentamientoResultado consultarAsentamientos (AsentamientoResultado resultado){
		
		// Lista con los criterios que se usaran en la busqueda
		Map<String, Map<String, TipoCondicion>> listaMetodos = new HashMap<String, Map<String, TipoCondicion>>();
				
		// Aadimos las condiciones para el codigo del expediente
		Map<String, TipoCondicion> condicionCodigoExpediente = new HashMap<String, TipoCondicion>();
		condicionCodigoExpediente.put("ASE.STR_CODEXPEDIENTE", TipoCondicion.Igual);
		listaMetodos.put("getCodExpediente", condicionCodigoExpediente);
		
		// Aadimos las condiciones para el codigo del documento
		Map<String, TipoCondicion> condicionCodigoTramite = new HashMap<String, TipoCondicion>();
		condicionCodigoTramite.put("EXPE.STR_CODTRAMITE", TipoCondicion.Igual);
		listaMetodos.put("getCodTramite", condicionCodigoTramite);
		
		// Aadimos las condiciones para el codigo del asentamiento
		Map<String, TipoCondicion> condicionCodigoAsentamiento = new HashMap<String, TipoCondicion>();
		condicionCodigoAsentamiento.put("ASE.STR_CODASENTAMIENTO", TipoCondicion.Igual);
		listaMetodos.put("getCodAsentamiento", condicionCodigoAsentamiento);
		
		// Aadimos las condiciones para el estado
		Map<String, TipoCondicion> condicionEstado = new HashMap<String, TipoCondicion>();
		condicionEstado.put("ASE.IND_CODESTADO", TipoCondicion.Igual);
		listaMetodos.put("getCodEstado", condicionEstado);
		
		// Aadimos las condicones para la fecha inicio presentacion
		Map<String, TipoCondicion> condicionFechaAltaInicio = new HashMap<String, TipoCondicion>();
		condicionFechaAltaInicio.put("ASE.FEC_FECHAPRESENTACION", TipoCondicion.MayorIgual);
		listaMetodos.put("getFechaAltaIni", condicionFechaAltaInicio);
		
		// Aadimos las condicones para la fecha final presentacion
		Map<String, TipoCondicion> condicionFechaAltaFinal = new HashMap<String, TipoCondicion>();
		condicionFechaAltaFinal.put("ASE.FEC_FECHAPRESENTACION", TipoCondicion.MenorIgual);
		listaMetodos.put("getFechaAltaFinal", condicionFechaAltaFinal);
		
		// Aadimos las condicones para la fecha desde
		Map<String, TipoCondicion> condicionFechaDocInicio = new HashMap<String, TipoCondicion>();
		condicionFechaDocInicio.put("ASE.FEC_FECHADOCUMENTO", TipoCondicion.MayorIgual);
		listaMetodos.put("getFechaDocIni", condicionFechaDocInicio);
		
		// Aadimos las condicones para la fecha hasta
		Map<String, TipoCondicion> condicionFechaDocFinal = new HashMap<String, TipoCondicion>();
		condicionFechaDocFinal.put("ASE.FEC_FECHADOCUMENTO", TipoCondicion.MenorIgual);
		listaMetodos.put("getFechaDocFinal", condicionFechaDocFinal);
		
		List<Asentamiento> listaAsentamientos = daoAsentamiento.consultaAsentamientosPorCriterios(resultado.getAsentamiento(), listaMetodos);
		
		resultado.setListaAsentamientos(listaAsentamientos);
		resultado.setContador(listaAsentamientos.size());
		
		return resultado;
	}
	
	//Busca Asentamientos
	public AsentamientoResultado consultarAsentamientosOr (AsentamientoResultado resultado){
		
		// Lista con los criterios que se usaran en la busqueda
		Map<String, Map<String, TipoCondicion>> listaMetodos = new HashMap<String, Map<String, TipoCondicion>>();
				
		// Aadimos las condiciones para el codigo del expediente
		Map<String, TipoCondicion> condicionCodigoExpediente = new HashMap<String, TipoCondicion>();
		condicionCodigoExpediente.put("ASE.STR_CODEXPEDIENTE", TipoCondicion.Igual);
		listaMetodos.put("getCodExpediente", condicionCodigoExpediente);
		
		// Aadimos las condiciones para el codigo del documento
		Map<String, TipoCondicion> condicionCodigoTramite = new HashMap<String, TipoCondicion>();
		condicionCodigoTramite.put("EXPE.STR_CODTRAMITE", TipoCondicion.Igual);
		listaMetodos.put("getCodTramite", condicionCodigoTramite);
		
		// Aadimos las condiciones para el codigo del asentamiento
		Map<String, TipoCondicion> condicionCodigoAsentamiento = new HashMap<String, TipoCondicion>();
		condicionCodigoAsentamiento.put("ASE.STR_CODASENTAMIENTO", TipoCondicion.Igual);
		listaMetodos.put("getCodAsentamiento", condicionCodigoAsentamiento);
		
		// Aadimos las condicones para la fecha inicio presentacion
		Map<String, TipoCondicion> condicionFechaAltaInicio = new HashMap<String, TipoCondicion>();
		condicionFechaAltaInicio.put("ASE.FEC_FECHAPRESENTACION", TipoCondicion.MayorIgual);
		listaMetodos.put("getFechaAltaIni", condicionFechaAltaInicio);
		
		// Aadimos las condicones para la fecha final presentacion
		Map<String, TipoCondicion> condicionFechaAltaFinal = new HashMap<String, TipoCondicion>();
		condicionFechaAltaFinal.put("ASE.FEC_FECHAPRESENTACION", TipoCondicion.MenorIgual);
		listaMetodos.put("getFechaAltaFinal", condicionFechaAltaFinal);
		
		// Aadimos las condicones para la fecha desde
		Map<String, TipoCondicion> condicionFechaDocInicio = new HashMap<String, TipoCondicion>();
		condicionFechaDocInicio.put("ASE.FEC_FECHADOCUMENTO", TipoCondicion.MayorIgual);
		listaMetodos.put("getFechaDocIni", condicionFechaDocInicio);
		
		// Aadimos las condicones para la fecha hasta
		Map<String, TipoCondicion> condicionFechaDocFinal = new HashMap<String, TipoCondicion>();
		condicionFechaDocFinal.put("ASE.FEC_FECHADOCUMENTO", TipoCondicion.MenorIgual);
		listaMetodos.put("getFechaDocFinal", condicionFechaDocFinal);
		
		List<Asentamiento> listaAsentamientos = daoAsentamiento.consultaAsentamientosPorCriteriosOr(resultado.getAsentamiento(), listaMetodos);
		
		resultado.setListaAsentamientos(listaAsentamientos);
		resultado.setContador(listaAsentamientos.size());
		
		return resultado;
	}
	
	//Busca Asentamientos
	public AsentamientoResultado consultarAsentamientosNot (AsentamientoResultado resultado){
		
		// Lista con los criterios que se usaran en la busqueda
		Map<String, Map<String, TipoCondicion>> listaMetodos = new HashMap<String, Map<String, TipoCondicion>>();
				
		// Aadimos las condiciones para el codigo del expediente
		Map<String, TipoCondicion> condicionCodigoExpediente = new HashMap<String, TipoCondicion>();
		condicionCodigoExpediente.put("ASE.STR_CODEXPEDIENTE", TipoCondicion.Distinto);
		listaMetodos.put("getCodExpediente", condicionCodigoExpediente);
		
		// Aadimos las condiciones para el codigo del documento
		Map<String, TipoCondicion> condicionCodigoTramite = new HashMap<String, TipoCondicion>();
		condicionCodigoTramite.put("EXPE.STR_CODTRAMITE", TipoCondicion.Distinto);
		listaMetodos.put("getCodTramite", condicionCodigoTramite);
		
		// Aadimos las condiciones para el codigo del asentamiento
		Map<String, TipoCondicion> condicionCodigoAsentamiento = new HashMap<String, TipoCondicion>();
		condicionCodigoAsentamiento.put("ASE.STR_CODASENTAMIENTO", TipoCondicion.Distinto);
		listaMetodos.put("getCodAsentamiento", condicionCodigoAsentamiento);
		
		// Aadimos las condiciones para el estado
		Map<String, TipoCondicion> condicionEstado = new HashMap<String, TipoCondicion>();
		condicionEstado.put("ASE.IND_CODESTADO", TipoCondicion.Igual);
		listaMetodos.put("getCodEstado", condicionEstado);
		
		// Aadimos las condicones para la fecha inicio presentacion
		Map<String, TipoCondicion> condicionFechaAltaInicio = new HashMap<String, TipoCondicion>();
		condicionFechaAltaInicio.put("ASE.FEC_FECHAPRESENTACION", TipoCondicion.MenorIgual);
		listaMetodos.put("getFechaAltaIni", condicionFechaAltaInicio);
		
		// Aadimos las condicones para la fecha final presentacion
		Map<String, TipoCondicion> condicionFechaAltaFinal = new HashMap<String, TipoCondicion>();
		condicionFechaAltaFinal.put("ASE.FEC_FECHAPRESENTACION", TipoCondicion.MayorIgual);
		listaMetodos.put("getFechaAltaFinal", condicionFechaAltaFinal);
		
		// Aadimos las condicones para la fecha desde
		Map<String, TipoCondicion> condicionFechaDocInicio = new HashMap<String, TipoCondicion>();
		condicionFechaDocInicio.put("ASE.FEC_FECHADOCUMENTO", TipoCondicion.MenorIgual);
		listaMetodos.put("getFechaDocIni", condicionFechaDocInicio);
		
		// Aadimos las condicones para la fecha hasta
		Map<String, TipoCondicion> condicionFechaDocFinal = new HashMap<String, TipoCondicion>();
		condicionFechaDocFinal.put("ASE.FEC_FECHADOCUMENTO", TipoCondicion.MayorIgual);
		listaMetodos.put("getFechaDocFinal", condicionFechaDocFinal);
		
		List<Asentamiento> listaAsentamientos = daoAsentamiento.consultaAsentamientosPorCriterios(resultado.getAsentamiento(), listaMetodos);
		
		resultado.setListaAsentamientos(listaAsentamientos);
		resultado.setContador(listaAsentamientos.size());
		
		return resultado;
	}
	
	@Override
	public byte[] obtenerCSVAsentamientos(String[] elegidos) {

		byte[] archivoCSV = null;

		if (elegidos == null || elegidos.length == 0) {
			throw new CronosValidacionException("es.alten.diputacion.add.servicios.generar.vacio");
		}
		
		List<Asentamiento> asentamientos = new ArrayList <Asentamiento>();


		for(String elegido : elegidos){
			Asentamiento asentamiento = daoAsentamiento.consultaAsentamientoPorCodigo(elegido);
			asentamientos.add(asentamiento);
		}
		
		archivoCSV = CSVAddUtils.obtenerCSVAsentamientos(asentamientos);

		return archivoCSV;
	}
	
	@Override
	public void rechazoAsentamiento(Asentamiento asentamiento){
		
		int asentamientoRechazado = daoAsentamiento.rechazarAsentamiento(asentamiento);
		
		if(asentamientoRechazado > 0){ } else{
	    	throw new CronosAbstraccionException("es.alten.diputacion.add.servicios.rechazar.asentamiento");
	    }
	}
	


	@Override
	public Asentamiento consultaAsentamiento(AsentamientoForm asentamientoForm) {
		Asentamiento asentamiento = daoAsentamiento.consultaAsentamientoPorCodigo(asentamientoForm.getCodAsentamiento());
		return asentamiento;
	}
	
	@Override
	public Persona consultaInteresadoByCodigo( String codInteresado ) {
		Persona persona = personaDAO.consultaPersonaByCodigo(codInteresado);
		return persona;
	}
	
	@Override
	public int guardarAsentamiento(Asentamiento asentamiento) {
		int resultado = daoAsentamiento.guardarAsentamiento(asentamiento);
		if(resultado > 0){return resultado; } else{
	    	throw new CronosAbstraccionException("es.alten.diputacion.add.servicios.rechazar.asentamiento");
	    }
		
	}
	
	//Valida el asentanmiento
	@Override
	public AsentamientoResultado validarAsentamiento(DocumentoPendienteForm asentamientoForm) {
		@SuppressWarnings("unused")
		Logger log = Logger.getLogger(AsentamientoServicioImpl.class);
		AsentamientoResultado resultado = new AsentamientoResultado(); 
		
		//TODO FALTA VALIDAR

		resultado.setConfirmacion(ConfirmacionAsentamiento.CORRECTO);
		resultado.setAsentamiento(asentamientoForm);

		return resultado;
	}

}
