package es.alten.diputacion.add.servicios.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.Logger;
import org.springframework.dao.DataAccessException;
import org.springframework.transaction.annotation.Transactional;


import es.alten.diputacion.add.abstraccion.SubtipologiaDao;
import es.alten.diputacion.add.abstraccion.TipologiaDao;
import es.alten.diputacion.add.abstraccion.TramiteDao;
import es.alten.diputacion.add.excepcion.CronosValidacionException;
import es.alten.diputacion.add.modelo.Subtipologia;
import es.alten.diputacion.add.modelo.Tipologia;
import es.alten.diputacion.add.modelo.Tramite;
import es.alten.diputacion.add.modelo.TramiteExpandido;
import es.alten.diputacion.add.modelo.resultado.TramiteExpandidoResultado;
import es.alten.diputacion.add.modelo.resultado.TramiteResultado;
import es.alten.diputacion.add.modelo.resultado.TramiteResultado.ConfirmacionTramite;
import es.alten.diputacion.add.modelo.struts.TramiteForm;
import es.alten.diputacion.add.servicios.TramiteServicio;
import es.alten.diputacion.add.util.CSVAddUtils;
import es.alten.diputacion.add.util.TipoCondicion;

@Transactional(rollbackFor = DataAccessException.class)
public class TramiteServicioImpl implements TramiteServicio{
	
	private static final Log logger = LogFactory.getLog(TramiteServicioImpl.class);
	
	private TramiteDao tramiteDAO;
	private SubtipologiaDao subtipologiaDAO;
	private TipologiaDao tipologiaDAO;
	
	public void setTramiteDAO(TramiteDao tramiteDAO) {
		this.tramiteDAO = tramiteDAO;
	}

	public TramiteDao getTramiteDAO() {
		return tramiteDAO;
	}

	public SubtipologiaDao getSubtipologiaDAO() {
		return subtipologiaDAO;
	}

	public void setSubtipologiaDAO(SubtipologiaDao subtipologiaDAO) {
		this.subtipologiaDAO = subtipologiaDAO;
	}

	public TipologiaDao getTipologiaDAO() {
		return tipologiaDAO;
	}

	public void setTipologiaDAO(TipologiaDao tipologiaDAO) {
		this.tipologiaDAO = tipologiaDAO;
	}

	@Override
	public TramiteExpandidoResultado consultarTramites (TramiteExpandido tramiteExpandido){
		
		// Lista con los criterios que se usaran en la busqueda
		Map<String, Map<String, TipoCondicion>> listaMetodos = new HashMap<String, Map<String, TipoCondicion>>();
				
		// Aadimos las condiciones para el codigod el tramite
		Map<String, TipoCondicion> condicionCodigoTramite = new HashMap<String, TipoCondicion>();
		condicionCodigoTramite.put("ADD_TRAMITE.STR_CODTRAMITE", TipoCondicion.Igual);
		listaMetodos.put("getCodigoTramite", condicionCodigoTramite);
		
		// Aadimos las condiciones para la descricion del tramite
		Map<String, TipoCondicion> condicionDenominacion = new HashMap<String, TipoCondicion>();
		condicionDenominacion.put("ADD_TRAMITE.STR_DESCTRAMITE", TipoCondicion.Igual);
		listaMetodos.put("getDescTramite", condicionDenominacion);
		
		// Aadimos las condiciones para la clasificacion del archivo
		Map<String, TipoCondicion> condicionClasificacion = new HashMap<String, TipoCondicion>();
		condicionClasificacion.put("ADD_TRAMITE.STR_CLASIFICACIONARCHIVO", TipoCondicion.Igual);
		listaMetodos.put("getClasificacionArchivo", condicionClasificacion);
		
		// Aadimos las condiciones para la Subtipologia
		Map<String, TipoCondicion> condicionSubtipologia = new HashMap<String, TipoCondicion>();
		condicionSubtipologia.put("ADD_TRAMITE.STR_CODSUBTIPOLOGIA", TipoCondicion.Igual);
		listaMetodos.put("getCodSubtipologia", condicionSubtipologia);		
		
		// Aadimos las condiciones para la Familia
		Map<String, TipoCondicion> condicionFamilia = new HashMap<String, TipoCondicion>();
		condicionFamilia.put("ADD_TRAMITE.STR_FAMILIA", TipoCondicion.Igual);
		listaMetodos.put("getFamilia", condicionFamilia);
		
		// Aadimos las condiciones para la SubFamilia
		Map<String, TipoCondicion> condicionSubfamilia = new HashMap<String, TipoCondicion>();
		condicionSubfamilia.put("ADD_TRAMITE.STR_SUBFAMILIA", TipoCondicion.Igual);
		listaMetodos.put("getSubfamilia", condicionSubfamilia);
		
		// Aadimos las condiciones para el Area
		Map<String, TipoCondicion> condicionArea = new HashMap<String, TipoCondicion>();
		condicionArea.put("ADD_TRAMITE.STR_AREA", TipoCondicion.Igual);
		listaMetodos.put("getArea", condicionArea);
		
		// Aadimos las condiciones para el Tema
		Map<String, TipoCondicion> condicionTema = new HashMap<String, TipoCondicion>();
		condicionTema.put("ADD_TRAMITE.STR_TEMA", TipoCondicion.Igual);
		listaMetodos.put("getTema", condicionTema);
		
		// Aadimos las condiciones para la Admin Componentes
		Map<String, TipoCondicion> condicionAdministracionComponentes = new HashMap<String, TipoCondicion>();
		condicionAdministracionComponentes.put("ADD_TRAMITE.STR_ADMINISTRACIONCOMPONENTE", TipoCondicion.Igual);
		listaMetodos.put("getAdministracionComponente", condicionAdministracionComponentes);
		
		// Aadimos las condiciones para la UOrganizativaDestino
		Map<String, TipoCondicion> condicionUOrganizativaDestino = new HashMap<String, TipoCondicion>();
		condicionUOrganizativaDestino.put("ADD_TRAMITE.STR_CODUORGANIZATIVADESTINO", TipoCondicion.Igual);
		listaMetodos.put("getCodUOrganizativaDestino", condicionUOrganizativaDestino);
		
		// Aadimos las condiciones para verificar que el tramite no este eliminado
		tramiteExpandido.setEliminado("N");
		Map<String, TipoCondicion> condicionTramiteEliminado = new HashMap<String, TipoCondicion>();
		condicionTramiteEliminado.put("ADD_TRAMITE.IND_ELIMINADO", TipoCondicion.Igual);
		listaMetodos.put("getEliminado", condicionTramiteEliminado);
		
		List<TramiteExpandido> listaTramites = tramiteDAO.consultaTramitesPorCriterios(tramiteExpandido, listaMetodos);
		
		for (Tramite tramite : listaTramites){
			
			Subtipologia subtipologia = subtipologiaDAO.consultaSubtipologiaPorTramite(tramite.getCodSubtipologia());
			tramite.setSubtipologia(subtipologia);
			Tipologia tipologia = tipologiaDAO.consultaTipologiaPorSubtipologia(subtipologia.getCodTipologia());
			tramite.setTipologia(tipologia);	
		}
		
		TramiteExpandidoResultado tramiteExpandidoResultado = new TramiteExpandidoResultado();
		tramiteExpandidoResultado.setListaTramites(listaTramites);
		tramiteExpandidoResultado.setContador(listaTramites.size());
		
		return tramiteExpandidoResultado;
	}
	
	@Override
	public TramiteExpandidoResultado consultarTramitesOr (TramiteExpandido tramiteExpandido){
				
		// Lista con los criterios que se usaran en la busqueda
		Map<String, Map<String, TipoCondicion>> listaMetodos = new HashMap<String, Map<String, TipoCondicion>>();
				
		// Aadimos las condiciones para el codigo del expediente
		Map<String, TipoCondicion> condicionCodigoTramite = new HashMap<String, TipoCondicion>();
		condicionCodigoTramite.put("ADD_TRAMITE.STR_CODTRAMITE", TipoCondicion.Igual);
		listaMetodos.put("getCodigoTramite", condicionCodigoTramite);
		
		// Aadimos las condiciones para el codigo del documento
		Map<String, TipoCondicion> condicionDenominacion = new HashMap<String, TipoCondicion>();
		condicionDenominacion.put("ADD_TRAMITE.STR_DESCTRAMITE", TipoCondicion.Igual);
		listaMetodos.put("getDescTramite", condicionDenominacion);
		
		// Aadimos las condiciones para el codigo del documento
		Map<String, TipoCondicion> condicionClasificacion = new HashMap<String, TipoCondicion>();
		condicionClasificacion.put("ADD_TRAMITE.STR_CLASIFICACIONARCHIVO", TipoCondicion.Igual);
		listaMetodos.put("getClasificacionArchivo", condicionClasificacion);
		
		// Aadimos las condiciones para la Subtipologia
		Map<String, TipoCondicion> condicionSubtipologia = new HashMap<String, TipoCondicion>();
		condicionSubtipologia.put("ADD_TRAMITE.STR_CODSUBTIPOLOGIA", TipoCondicion.Igual);
		listaMetodos.put("getCodSubtipologia", condicionSubtipologia);		
		
		// Aadimos las condiciones para la Familia
		Map<String, TipoCondicion> condicionFamilia = new HashMap<String, TipoCondicion>();
		condicionFamilia.put("ADD_TRAMITE.STR_FAMILIA", TipoCondicion.Igual);
		listaMetodos.put("getFamilia", condicionFamilia);
		
		// Aadimos las condiciones para la SubFamilia
		Map<String, TipoCondicion> condicionSubfamilia = new HashMap<String, TipoCondicion>();
		condicionSubfamilia.put("ADD_TRAMITE.STR_SUBFAMILIA", TipoCondicion.Igual);
		listaMetodos.put("getSubfamilia", condicionSubfamilia);
		
		// Aadimos las condiciones para el Area
		Map<String, TipoCondicion> condicionArea = new HashMap<String, TipoCondicion>();
		condicionArea.put("ADD_TRAMITE.STR_AREA", TipoCondicion.Igual);
		listaMetodos.put("getArea", condicionArea);
		
		// Aadimos las condiciones para el Tema
		Map<String, TipoCondicion> condicionTema = new HashMap<String, TipoCondicion>();
		condicionTema.put("ADD_TRAMITE.STR_TEMA", TipoCondicion.Igual);
		listaMetodos.put("getTema", condicionTema);
		
		// Aadimos las condiciones para la Admin Componentes
		Map<String, TipoCondicion> condicionAdministracionComponentes = new HashMap<String, TipoCondicion>();
		condicionAdministracionComponentes.put("ADD_TRAMITE.STR_ADMINISTRACIONCOMPONENTE", TipoCondicion.Igual);
		listaMetodos.put("getAdministracionComponente", condicionAdministracionComponentes);
		
		// Aadimos las condiciones para la UOrganizativaDestino
		Map<String, TipoCondicion> condicionUOrganizativaDestino = new HashMap<String, TipoCondicion>();
		condicionUOrganizativaDestino.put("ADD_TRAMITE.STR_CODUORGANIZATIVADESTINO", TipoCondicion.Igual);
		listaMetodos.put("getCodUOrganizativaDestino", condicionUOrganizativaDestino);
		
		// Aadimos las condiciones para verificar que el tramite no este eliminado
		tramiteExpandido.setEliminado("N");
		Map<String, TipoCondicion> condicionTramiteEliminado = new HashMap<String, TipoCondicion>();
		condicionTramiteEliminado.put("ADD_TRAMITE.IND_ELIMINADO", TipoCondicion.Igual);
		listaMetodos.put("getEliminado", condicionTramiteEliminado);
		
		List<TramiteExpandido> listaTramites = tramiteDAO.consultaTramitesPorCriteriosOr(tramiteExpandido, listaMetodos);
		
		for (Tramite tramite : listaTramites){
			
			Subtipologia subtipologia = subtipologiaDAO.consultaSubtipologiaPorTramite(tramite.getCodSubtipologia());
			tramite.setSubtipologia(subtipologia);
			Tipologia tipologia = tipologiaDAO.consultaTipologiaPorSubtipologia(subtipologia.getCodTipologia());
			tramite.setTipologia(tipologia);	
		}
		
		TramiteExpandidoResultado tramiteExpandidoResultado = new TramiteExpandidoResultado();
		tramiteExpandidoResultado.setListaTramites(listaTramites);
		tramiteExpandidoResultado.setContador(listaTramites.size());
		
		return tramiteExpandidoResultado;
	}
	
	@Override
	public TramiteExpandidoResultado consultarTramitesNot (TramiteExpandido tramiteExpandido){
		
		// Lista con los criterios que se usaran en la busqueda
		Map<String, Map<String, TipoCondicion>> listaMetodos = new HashMap<String, Map<String, TipoCondicion>>();
				
		// Aadimos las condiciones para el codigo del expediente
		Map<String, TipoCondicion> condicionCodigoTramite = new HashMap<String, TipoCondicion>();
		condicionCodigoTramite.put("ADD_TRAMITE.STR_CODTRAMITE", TipoCondicion.Distinto);
		listaMetodos.put("getCodigoTramite", condicionCodigoTramite);
		
		// Aadimos las condiciones para el codigo del documento
		Map<String, TipoCondicion> condicionDenominacion = new HashMap<String, TipoCondicion>();
		condicionDenominacion.put("ADD_TRAMITE.STR_DESCTRAMITE", TipoCondicion.Distinto);
		listaMetodos.put("getDescTramite", condicionDenominacion);
		
		// Aadimos las condiciones para el codigo del documento
		Map<String, TipoCondicion> condicionClasificacion = new HashMap<String, TipoCondicion>();
		condicionClasificacion.put("ADD_TRAMITE.STR_CLASIFICACIONARCHIVO", TipoCondicion.Distinto);
		listaMetodos.put("getClasificacionArchivo", condicionClasificacion);	
		
		// Aadimos las condiciones para la Subtipologia
		Map<String, TipoCondicion> condicionSubtipologia = new HashMap<String, TipoCondicion>();
		condicionSubtipologia.put("ADD_TRAMITE.STR_CODSUBTIPOLOGIA", TipoCondicion.Igual);
		listaMetodos.put("getCodSubtipologia", condicionSubtipologia);		
		
		// Aadimos las condiciones para la Familia
		Map<String, TipoCondicion> condicionFamilia = new HashMap<String, TipoCondicion>();
		condicionFamilia.put("ADD_TRAMITE.STR_FAMILIA", TipoCondicion.Igual);
		listaMetodos.put("getFamilia", condicionFamilia);
		
		// Aadimos las condiciones para la SubFamilia
		Map<String, TipoCondicion> condicionSubfamilia = new HashMap<String, TipoCondicion>();
		condicionSubfamilia.put("ADD_TRAMITE.STR_SUBFAMILIA", TipoCondicion.Igual);
		listaMetodos.put("getSubfamilia", condicionSubfamilia);
		
		// Aadimos las condiciones para el Area
		Map<String, TipoCondicion> condicionArea = new HashMap<String, TipoCondicion>();
		condicionArea.put("ADD_TRAMITE.STR_AREA", TipoCondicion.Igual);
		listaMetodos.put("getArea", condicionArea);
		
		// Aadimos las condiciones para el Tema
		Map<String, TipoCondicion> condicionTema = new HashMap<String, TipoCondicion>();
		condicionTema.put("ADD_TRAMITE.STR_TEMA", TipoCondicion.Igual);
		listaMetodos.put("getTema", condicionTema);
		
		// Aadimos las condiciones para la Admin Componentes
		Map<String, TipoCondicion> condicionAdministracionComponentes = new HashMap<String, TipoCondicion>();
		condicionAdministracionComponentes.put("ADD_TRAMITE.STR_ADMINISTRACIONCOMPONENTE", TipoCondicion.Igual);
		listaMetodos.put("getAdministracionComponente", condicionAdministracionComponentes);
		
		// Aadimos las condiciones para la UOrganizativaDestino
		Map<String, TipoCondicion> condicionUOrganizativaDestino = new HashMap<String, TipoCondicion>();
		condicionUOrganizativaDestino.put("ADD_TRAMITE.STR_CODUORGANIZATIVADESTINO", TipoCondicion.Igual);
		listaMetodos.put("getCodUOrganizativaDestino", condicionUOrganizativaDestino);
		
		// Aadimos las condiciones para verificar que el tramite no este eliminado
		tramiteExpandido.setEliminado("N");
		Map<String, TipoCondicion> condicionTramiteEliminado = new HashMap<String, TipoCondicion>();
		condicionTramiteEliminado.put("ADD_TRAMITE.IND_ELIMINADO", TipoCondicion.Igual);
		listaMetodos.put("getEliminado", condicionTramiteEliminado);
		
		List<TramiteExpandido> listaTramites = tramiteDAO.consultaTramitesPorCriterios(tramiteExpandido, listaMetodos);
		
		for (Tramite tramite : listaTramites){
			
			Subtipologia subtipologia = subtipologiaDAO.consultaSubtipologiaPorTramite(tramite.getCodSubtipologia());
			tramite.setSubtipologia(subtipologia);
			Tipologia tipologia = tipologiaDAO.consultaTipologiaPorSubtipologia(subtipologia.getCodTipologia());
			tramite.setTipologia(tipologia);	
		}
		
		TramiteExpandidoResultado tramiteExpandidoResultado = new TramiteExpandidoResultado();
		tramiteExpandidoResultado.setListaTramites(listaTramites);
		tramiteExpandidoResultado.setContador(listaTramites.size());
		
		return tramiteExpandidoResultado;
	}
	
	@Override
	public Tramite recuperarTramiteByCodigo (String elegido){
		
		Tramite tramite;
		try {
			tramite = tramiteDAO.consultaTramitePorCodigo(elegido);
		} catch (Exception e) {
			logger.error(e);
			throw new CronosValidacionException("es.alten.diputacion.add.servicios.recuperar.tramites");
		}		
		return tramite;
	}
	
	@Override
	public byte[] obtenerCSVTramites(String[] elegidos) {

		byte[] archivoCSV = null;

		if (elegidos == null || elegidos.length == 0) {
			throw new CronosValidacionException("es.alten.diputacion.add.servicios.generar.vacio");
		}
		
		List<Tramite> tramites = new ArrayList <Tramite>();


		for(String elegido : elegidos){
			Tramite tramite = tramiteDAO.consultaTramitePorCodigo(elegido);
			tramites.add(tramite);
		}
		
		archivoCSV = CSVAddUtils.obtenerCSVTramite(tramites);

		return archivoCSV;
	}
	
	@Override
	public TramiteResultado validarTramite(TramiteForm tramiteForm) {
		@SuppressWarnings("unused")
		Logger log = Logger.getLogger(TramiteServicioImpl.class);
		TramiteResultado resultado = new TramiteResultado(); 
		
		//TODO FALTA VALIDAR

		resultado.setConfirmacion(ConfirmacionTramite.CORRECTO);
		resultado.setTramite(tramiteForm);

		return resultado;
	}

	@Override
	public List<Tramite> getTramitesByCodSubtipologia(String codSubtipologia) {
		List<Tramite> tramites = tramiteDAO.consultaTramitesByCodigoSubtipologia(codSubtipologia);
		return tramites;
	}

	@Override
	public String validarTramiteExpandido(TramiteExpandido tramiteExpandido) {
		// FIXME AQUI SE DEBERIAN VALIDAR LOS DATOS
		return "CORRECTO";
	}

}
