package es.alten.diputacion.add.servicios.impl;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import org.apache.log4j.Logger;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import es.alten.diputacion.add.abstraccion.EntidadDao;
import es.alten.diputacion.add.abstraccion.UOrganizativaDao;
import es.alten.diputacion.add.aspectos.Historico;
import es.alten.diputacion.add.modelo.Entidad;
import es.alten.diputacion.add.modelo.UOrganizativa;
import es.alten.diputacion.add.modelo.resultado.ResultadoUOPorEntidad;
import es.alten.diputacion.add.servicios.UOrganizativaServicio;
import es.alten.diputacion.add.servicios.request.InActivacionUOrganicas;
import es.alten.diputacion.add.servicios.request.InAltaUO;

@Transactional
public class UOrganizativaServicioImpl implements UOrganizativaServicio {
	private Logger log = Logger.getLogger(UOrganizativaServicioImpl.class);
	private UOrganizativaDao daoUOrganizativa;
	private EntidadDao daoEntidad;

	@Override
	public List<ResultadoUOPorEntidad> consultaTodasUnidadesOrganicas() {
		List <ResultadoUOPorEntidad>listaResultadoUO = new ArrayList<ResultadoUOPorEntidad>();
		List<UOrganizativa> listaUO = daoUOrganizativa.consultarTodasUOrganizativas();
		for(UOrganizativa uo: listaUO){
			ResultadoUOPorEntidad resultadoUO = new ResultadoUOPorEntidad();
			Entidad entidad = daoEntidad.consultarEntidadPorId(uo.getCodEntidad());
			resultadoUO.setEntidad(entidad);
			resultadoUO.setUo(uo);
			listaResultadoUO.add(resultadoUO);
		}
		return listaResultadoUO;
	}

	public UOrganizativaDao getDaoUOrganizativa() {
		return daoUOrganizativa;
	}
	public void setDaoUOrganizativa(UOrganizativaDao organizativaDao) {
		daoUOrganizativa = organizativaDao;
	}

	public EntidadDao getDaoEntidad() {
		return daoEntidad;
	}

	public void setDaoEntidad(EntidadDao daoEntidad) {
		this.daoEntidad = daoEntidad;
	}

	@Override
	public List<Entidad> consultaTodasEntidades() {
		return daoEntidad.consultarTodasEntidades();
	}

	@Override
	@Historico(codOperacion="ope2")
	@Transactional(propagation=Propagation.REQUIRED)
	public void altaUOrganica(InAltaUO in) {
		try {
			daoUOrganizativa.insertUnidadOrganica(in.getUnidadOrganica());
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}


	@Override
	@Historico(codOperacion="ope1")
	@Transactional(readOnly=true)
	public void activacionMasivaDeUOrganicas(InActivacionUOrganicas in) {
		for(String codUO : in.getListaUnidadesOrganizativas()){
			UOrganizativa unidadOrganizativa = daoUOrganizativa.selectUnidadOrganizativaPorCodUOrganizativa(codUO);
			if(in.isActivar()){
				unidadOrganizativa.setEstado(UOrganizativa.ACTIVADO);
			}
			else {
				unidadOrganizativa.setEstado(UOrganizativa.DESACTIVADO);
			}
			unidadOrganizativa.setFechaActivacion(new Date());
			unidadOrganizativa.setFechaBaja(null);
			log.info("\n##############  ANTES DE HACER EL UPDATE");
			daoUOrganizativa.updateUnidadOrganica(unidadOrganizativa);
			log.info("\n###############  DESPUES DE HACER EL UPDATE");
		}
		//Aqui se ejecuta el codigo del aspecto
	}
}
