package es.alten.diputacion.add.util;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.springframework.context.ApplicationContext;

import es.alten.diputacion.add.excepcion.CronosBaseException;
import es.alten.diputacion.add.excepcion.CronosValidacionException;

/**
 * Esta clase nos ofrece funcionalidad para iniciar el Contexto de Aplicacin
 * para Spring, manejar al usuario logado he insertar mensajes
 * 
 * @author jluis.martinez
 */
public class UtilidadesCronos {

	/** Contexto Spring para ejecucin de la aplicacin */
	public static ApplicationContext contextoSpring = null;
		
	public static ApplicationContext getContextoSpring() {
		return contextoSpring;
	}

	public static void setContextoSpring(final ApplicationContext _contextoSpring) {
		UtilidadesCronos.contextoSpring = _contextoSpring;
	}
	
	/** Lista de mensajes sobre la accin ejecutada */
	public static String listaMensajes = "mensajes";
	/** Constante con el nombre del Bean de usuario logado en la aplicacin */
	public static final String NombreUsuarioBean = "usuarioLogado";
	/** Utilizaremos el ThreadLocal para tener acceso al usuario actual */
	private static ThreadLocal<Object> usuarioLogado = new ThreadLocal<Object>();

	/** Nombre de la lista de los mensajes de error */
	public static String LISTA_MENSAJES = "mensajes";
	
	public static String LISTA_AVISOS = "avisos";

	private static ThreadLocal<es.alten.diputacion.add.excepcion.CronosBaseException> excepcionLanzada = new ThreadLocal<CronosBaseException>();

	// --- Recoge la Excepcion Lanzada del hilo actual.
	public static CronosBaseException getExcepcionLanzada() {
		if (excepcionLanzada.get() == null) {
			setExcepcionLanzada(new CronosValidacionException());
		}
		return excepcionLanzada.get();
	}

	// --- Pone la Excepcion Lanzada en el hilo actual.
	public static void setExcepcionLanzada(
			CronosBaseException cronosBaseException) {
		excepcionLanzada.set(cronosBaseException);
	}

	// Recoge usuario del hilo actual.
	public static Object getUsuarioLogado() {
		return usuarioLogado.get();
	}

	// Recoge usuario del hilo actual.
	/*public static Roles getRolUsuarioLogado() {
		return ((Usuario) usuarioLogado.get()).obtenerRolUsuario();
	}
*/
	// Recoge usuario del hilo actual.
	/*public static List<Unidad> getUnidadEnListaUsuarioLogado() {
		return ((Usuario) usuarioLogado.get()).getUnidadEnLista();
	}*/

	// Recoge usuario del hilo actual.
	/*public static Set<Rol> getRolEnListaUsuarioLogado() {
		return ((Usuario) usuarioLogado.get()).getRoles();
	}*/

	// Recoge usuario del hilo actual.
	/*public static Unidad getUnidadUsuarioLogado() {
		return ((Usuario) usuarioLogado.get()).getUnidad();
	}*/

	// Poner al usuario en el hilo actual.
	public static void setUsuarioLogado(Object usuario) {
		usuarioLogado.set(usuario);
	}

	/**
	 * 
	 * @param metadato
	 * @param clave
	 * @param parametros
	 */
	@SuppressWarnings("unchecked")
	public static void insertarMensaje(Map<String, Object> metadato,
			String clave, String... parametros) {
		Map<String, Object> mensajes;
		if (metadato.containsKey(listaMensajes)) {
			mensajes = (Map) metadato.get(listaMensajes);
		} else {
			mensajes = crearMap();
		}
		mensajes.put(clave, parametros);
		metadato.put(listaMensajes, mensajes);
	}

	/**
	 * 
	 * @param metadato
	 * @param clave
	 */
	@SuppressWarnings("unchecked")
	public static void insertarMensaje(Map<String, Object> metadato,
			String clave) {
		insertarMensaje(metadato, clave, (String[]) null);
	}

	/**
	 * 
	 * @param metadato
	 * @param mensajes
	 */
	@SuppressWarnings("unchecked")
	public static void insertarMensajes(Map<String, Object> metadato,
			Map<String, Object> mensajes) {
		Map<String, Object> mensajesAntiguos;
		if ((metadato != null) && metadato.containsKey(listaMensajes)) {
			mensajesAntiguos = (Map) metadato.get(listaMensajes);
			mensajes.putAll(mensajesAntiguos);
		}
		metadato.put(listaMensajes, mensajes);
	}

	/**
	 * 
	 * @param <K>
	 * @param <V>
	 * @return
	 */
	public static final <K, V> Map<K, V> crearMap() {
		return new HashMap<K, V>();
	}
	
	
	/**
	 * 
	 * @param metadato
	 * @param clave
	 * @param parametros
	 */
	@SuppressWarnings("unchecked")
	public static void insertarAviso(Map<String, Object> metadato,
			String clave, String aviso, String... parametros) {
		Map<String, Object> mensajes;
		if (metadato.containsKey(aviso)) {
			mensajes = (Map) metadato.get(aviso);
		} else {
			mensajes = crearMap();
		}
		mensajes.put(clave, parametros);
		metadato.put(aviso, mensajes);
	}

	/**
	 * 
	 * @param metadato
	 * @param clave
	 */
	@SuppressWarnings("unchecked")
	public static void insertarAviso(HttpServletRequest request,String clave, String aviso) {
		Map<String, Object> mensaje = UtilidadesCronos.crearMap();
		UtilidadesCronos.insertarAviso(mensaje, clave,aviso);
		request.setAttribute(aviso, mensaje.get(aviso));
	}
	
	/**
	 * 
	 * @param metadato
	 * @param clave
	 */
	@SuppressWarnings("unchecked")
	public static void insertarAviso(Map<String, Object> metadato,
			String clave, String aviso) {
		insertarAviso(metadato, clave,aviso, (String[]) null);
	}

	/**
	 * 
	 * @param metadato
	 * @param mensajes
	 */
	@SuppressWarnings("unchecked")
	public static void insertarAviso(Map<String, Object> metadato,
			Map<String, Object> mensajes, String aviso) {
		Map<String, Object> mensajesAntiguos;
		if ((metadato != null) && metadato.containsKey(aviso)) {
			mensajesAntiguos = (Map) metadato.get(aviso);
			mensajes.putAll(mensajesAntiguos);
		}
		metadato.put(aviso, mensajes);
	}
	
	
	
	
	public static Date getFechaActual(){
		Calendar fechaActual = Calendar.getInstance();
//		SimpleDateFormat formato = new SimpleDateFormat("yyyyMMdd");
//		String cadenaFecha = formato.format(fechaActual);
		
//		Calendar 
		
		int dia = fechaActual.get(Calendar.DATE);
		int mes = fechaActual.get(Calendar.MONTH);
		int annio = fechaActual.get(Calendar.YEAR);
		
		Date hoy = new GregorianCalendar(annio,mes,dia).getTime();
		return hoy;
	}
	
	public static Date getPrimerDiaAnioActual(int annio){
//		Calendar fechaActual = Calendar.getInstance();
//		int annio = fechaActual.get(Calendar.YEAR);
		
		Date hoy = new GregorianCalendar(annio,Calendar.JANUARY,1).getTime();
		return hoy;
	}
	
}
