package es.alten.cronos.test;

import java.io.File;
import java.util.Collection;
import java.util.Map;

import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.springframework.context.ApplicationContext;
import org.springframework.mock.web.MockServletContext;
import org.springframework.web.context.ContextLoader;
import org.springframework.web.context.ContextLoaderListener;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

import servletunit.struts.MockStrutsTestCase;

public abstract class CronosMockStrutsTestCase extends MockStrutsTestCase {

	public CronosMockStrutsTestCase(String n) {
		super(n);
	}

	protected void setUp() throws Exception {
		super.setUp();

		File file = new File("src/main/webapp");
		logger.info("Usando el contexto: " + file.getAbsolutePath());

		this.context.setContextDirectory(file);

		// initialize Spring
		MockServletContext sc = new MockServletContext("");

		sc.addInitParameter(ContextLoader.CONFIG_LOCATION_PARAM,
				getConfigLocationParam());
		ServletContextListener contextListener = new ContextLoaderListener();
		ServletContextEvent event = new ServletContextEvent(sc);
		contextListener.contextInitialized(event);

		// magic bridge to make StrutsTestCase aware of Spring's Context
		getSession()
				.getServletContext()
				.setAttribute(
						WebApplicationContext.ROOT_WEB_APPLICATION_CONTEXT_ATTRIBUTE,
						sc
								.getAttribute(WebApplicationContext.ROOT_WEB_APPLICATION_CONTEXT_ATTRIBUTE));

		transferirContexto(WebApplicationContextUtils
				.getRequiredWebApplicationContext(context));

		request.setRemoteHost("127.0.0.1");
	}

	@SuppressWarnings("unchecked")
	public <T> Collection<T> assertListaSession(Class<T> clazz, String nombreLista) {
		Collection<T> lista = (Collection<T>) getSession().getAttribute(nombreLista);
		assertTrue("La lista de " + nombreLista
				+ " no contiene datos en session.", lista.size() > 0);
		return lista;
	}

	@SuppressWarnings("unchecked")
	public <T> Collection<T> assertListaRequestOrdenada(Class<T> clazz,
			String nombreLista) {
		return assertListaRequest(clazz, nombreLista);
		// List lista = (List) request.getAttribute(nombreLista);
	}

	@SuppressWarnings("unchecked")
	public <T> Collection<T> assertListaRequest(Class<T> clazz, String nombreLista) {
		// No se puede comprobar el tipo de la lista (el casting no funciona)
		Collection lista = (Collection) request.getAttribute(nombreLista);
		assertNotNull("La lista " + nombreLista + " no existe en el request.",
				lista);
		assertTrue("La lista " + nombreLista
				+ " no contiene datos en el request.", lista.size() > 0);
		T elemento = (T) lista.iterator().next();
		assertTrue("La lista " + nombreLista
				+ " se esperaba que tuviera elementos instancias de "
				+ clazz.getName() + " y tiene " + elemento.getClass(),
				clazz.isInstance(elemento));
		return lista;
	}

	@SuppressWarnings("unchecked")
	public <T> void assertNoListaRequest(Class<T> clazz, String nombreLista) {
		// No se puede comprobar el tipo de la lista (el casting no funciona)
		Collection lista = (Collection) request.getAttribute(nombreLista);

		assertFalse("La lista " + nombreLista
				+ " contiene datos en el request.", (lista == null || lista
				.size() > 0));
	}

	@SuppressWarnings("unchecked")
	public <T> T assertRequest(Class<T> clazz, String nombreLista) {
		T item = (T) request.getAttribute(nombreLista);
		assertNotNull("El parametro " + nombreLista
				+ " no existe en el request", item);
		return item;
	}

	@SuppressWarnings("unchecked")
	public void assertMensajes(String... nombreLista) {
		Map mensajes = (Map) request.getAttribute("mensajes");

		assertNotNull("No se encuentra la lista de errores 'mensajes'",
				mensajes);
		assertTrue("Tiene que recibir " + nombreLista.length + " mensajes.",
				mensajes.size() == nombreLista.length);

		for (String mensaje : nombreLista) {
			assertTrue("El parametro mensaje no existe en la lista de errores"
					+ mensajes, mensajes.containsKey(mensaje));
			// Object value = mensajes.get(mensaje);
		}

	}

	@SuppressWarnings("unchecked")
	public void assertNoMensajes() {
		Map mensajes = (Map) request.getAttribute("mensajes");

		if (mensajes != null) {
			fail("Se han encontrado los errores " + mensajes);
		}

	}

	/**
	 * Aade al servlet-contex el fichero que contiene el contexto de spring e.g
	 * "/WEB-INF/test-context.xml";
	 * 
	 * @return El contexto de spring
	 */
	protected abstract String getConfigLocationParam();

	protected abstract void transferirContexto(
			ApplicationContext applicationContext);

}