package es.alten.cronos.add.abstraccion.jdbc;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.simple.SimpleJdbcCall;
import org.springframework.jdbc.core.support.AbstractLobCreatingPreparedStatementCallback;
import org.springframework.jdbc.support.lob.DefaultLobHandler;
import org.springframework.jdbc.support.lob.LobCreator;

import es.alten.cronos.add.abstraccion.model.Documento;

public class DiputacionJdbcDaoImpl implements DiputacionJdbcDao {

	private JdbcTemplate jdbcTemplate;

    public void setDataSource(DataSource dataSource) {
        this.jdbcTemplate = new JdbcTemplate(dataSource);
    }

	public Map<String, Object> ejecutaProcedimiento(String nombreProcedimiento, Map<String, Object> listaParametros){
		SimpleJdbcCall simpleJdbcCall = new SimpleJdbcCall(jdbcTemplate);
		simpleJdbcCall.setProcedureName(nombreProcedimiento);
		Map<String, Object> salida = simpleJdbcCall.execute(listaParametros);
		return salida;
	}

	public void insertarDocumento(final Documento documento) {
		final InputStream blobIs = new ByteArrayInputStream(documento.getContenido());
		String query = "INSERT INTO DOCUMENTOS (TITULO_DOC, DESCRIPCION_DOC, CONTENIDO_DOC, CODIGO_ENTIDAD) VALUES (?,?,?,?)";
		DefaultLobHandler lobhandler = new DefaultLobHandler();
		jdbcTemplate.execute(query, new AbstractLobCreatingPreparedStatementCallback(lobhandler) {
			protected void setValues(final PreparedStatement ps, final LobCreator lobCreator) throws SQLException {
				ps.setString(1, documento.getTitulo());
				ps.setString(2, documento.getDescripcion());
				lobCreator.setBlobAsBinaryStream(ps, 3, blobIs, (int)documento.getContenido().length);
				ps.setLong(4, documento.getCodigoEntidad());
			}
		});
	}

}
