package es.alten.diputacion.add.abstraccion;

import java.util.List;
import java.util.Map;

import es.alten.diputacion.add.modelo.Asentamiento;
import es.alten.diputacion.add.util.TipoCondicion;

public interface AsentamientoDao {
	/**
	 * Recupera un documento pendiente por clave primaria
	 * 
	 * @param id Clave primeria de un documento pendiente
	 * @return documento pendiente encontrado
	 */
	public Asentamiento consultaSentamientoPorId(String id);
	/**
	 * Inserta un nuevo documento pendiente
	 * @param documento pendiente que se prtende insertar
	 */
	public void insertAsentamiento(Asentamiento doc);
	
	/**
	 * Actualiza un documento pendiente 
	 * @param doc documento pendiente a actualizar
	 */
	public void updateAsentamiento(Asentamiento doc);
	/**
	 * Obtiene una lista de asentamientos por unidad organica
	 * @Param unidadOrganica Unidad organica por la cual se buscara un Asentamiento 
	 * @return lista de asentamientos vinculadas a una determinada unidad organica
	 */
	public List<Asentamiento>  consultaAsentamientoPorUnidadOrganica(String codUnidadOrganica);
	
	public List<Asentamiento> consultaAsentamientosPorCriterios(Asentamiento asentamiento, Map<String, Map<String, TipoCondicion>> listaMetodos);
	
	public List<Asentamiento> consultaAsentamientosPorCriteriosOr(Asentamiento asentamiento, Map<String, Map<String, TipoCondicion>> listaMetodos);
	
	public Asentamiento consultaAsentamientoPorCodigo(String elegido);	
	
	public int rechazarAsentamiento(Asentamiento asentamiento);
	
	public int guardarAsentamiento (Asentamiento asentamiento);
}

