package es.alten.diputacion.add.abstraccion.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import es.alten.diputacion.add.abstraccion.AsentamientoDao;
import es.alten.diputacion.add.modelo.Asentamiento;
import es.alten.diputacion.add.util.AddJdbcUtils;
import es.alten.diputacion.add.util.TipoCondicion;


public class AsentamientoDaoImpl  implements AsentamientoDao{
	private JdbcTemplate jdbcTemplate;

	public Asentamiento consultaSentamientoPorId(String id) {
				String sql = "SELECT * FROM ADD_DOCUMENTOPENDIENTE WHERE UPPER(STR_CODDOCUMENTOPENDIENTE) = ?";
				Asentamiento asentamiento = (Asentamiento) jdbcTemplate.queryForObject(sql,
						new Object[]{ id.toUpperCase() }, new AsentamientoDaoImpl.RowMapperAsentamiento());
			return asentamiento;
	}
	
	public void updateAsentamiento(Asentamiento doc){
		String sql="UPDATE ADD_DOCUMENTOPENDIENTE SET STR_CODDOCUMENTOPENDIENTE=?, FEC_FECHAREGISTRODOCPTE=?, FEC_FECHAACEPTACIONDOCPTE=?, FEC_FECHARECHAZODOCPTE=?, STR_CODUORGANIZATIVA=?, IND_CODESTADO=? WHERE STR_CODDOCUMENTOPENDIENTE=?";
		Object[] parametros = new Object[]{doc.getCodDocumentoPendiente(),
				doc.getFechaRegistroDocPte(),
				doc.getFechaAceptacionDocPte(),
				doc.getFechaRechazoDocPte(),
				doc.getCodUOrganizativa(),
				doc.getCodEstado(),
				doc.getCodDocumentoPendiente()};
		jdbcTemplate.update(sql,parametros);
	}
	
	public void insertAsentamiento(Asentamiento doc){
		String sql="insert into ADD_DOCUMENTOPENDIENTE (STR_CODDOCUMENTOPENDIENTE,FEC_FECHAREGISTRODOCPTE,FEC_FECHAACEPTACIONDOCPTE,FEC_FECHARECHAZODOCPTE,STR_CODUORGANIZATIVA,IND_CODESTADO)values(?,?,?,?,?,?)";
		Object[] parametros = new Object[]{
				doc.getCodDocumentoPendiente(),
				doc.getFechaRegistroDocPte(),
				doc.getFechaAceptacionDocPte(),
				doc.getFechaRechazoDocPte(),
				doc.getCodUOrganizativa(),
				doc.getCodEstado()};
		jdbcTemplate.update(sql,parametros);
	}

	public JdbcTemplate getJdbcTemplate() {
		return jdbcTemplate;
	}

	public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
	}
	
	
	@SuppressWarnings("unchecked")
	public List<Asentamiento> consultaAsentamientoPorUnidadOrganica(String codUnidadOrganica){
		
		try {
			String sql = "SELECT * FROM ADD_ASENTAMIENTO WHERE STR_CODUORGANIZATIVA = ? AND IND_RECHAZADO= 'N'";
			Object[] parametros = new Object[]{codUnidadOrganica};
			List<Asentamiento> listaAsentamiento = jdbcTemplate.query(sql, parametros, new AsentamientoMapperGeneral());
			return listaAsentamiento;
		}
		catch(EmptyResultDataAccessException e){
			return new ArrayList<Asentamiento>();
		}	
	}
	
	private class AsentamientoMapperGeneral implements RowMapper {
		 public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
			
	    	Asentamiento asentamiento = new Asentamiento();
	    	asentamiento.setCodAsentamiento(rs.getString("STR_CODASENTAMIENTO"));
	    	asentamiento.setAsunto(rs.getString("STR_ASUNTO"));
	    	asentamiento.setFechaRegistro(rs.getDate("FEC_FECHAREGISTRO"));
	    	asentamiento.setFechaDocumento(rs.getDate("FEC_FECHADOCUMENTO"));
	    	asentamiento.setFechaPresentacion(rs.getDate("FEC_FECHAPRESENTACION"));
	    	asentamiento.setFechaAceptacion(rs.getDate("FEC_FECHAACEPTACION"));
	    	asentamiento.setFechaRechazo(rs.getDate("FEC_FECHARECHAZO"));
	    	asentamiento.setTipoAsentamiento(rs.getString("STR_TIPOASENTAMIENTO"));
	    	asentamiento.setViaPresentacion(rs.getString("STR_VIAPRESENTACION"));
	    	asentamiento.setSoporteFisico(rs.getString("STR_SOPORTEFISICO"));
	    	asentamiento.setUrgente(rs.getString("IND_URGENTE"));
	    	asentamiento.setFaltaDocEscaner(rs.getString("IND_FALTADOCESCANER"));
	    	asentamiento.setObservaciones(rs.getString("STR_OBSERVACIONES"));
	    	asentamiento.setCodUOrganizativa(rs.getString("STR_CODUORGANIZATIVA"));
	    	asentamiento.setCodEstado(rs.getString("IND_CODESTADO"));
	    	asentamiento.setCodExpediente(rs.getString("STR_CODEXPEDIENTE"));
	    	asentamiento.setCodInteresado(rs.getString("STR_CODINTERESADO"));
	    	asentamiento.setMotivoRechazo(rs.getString("STR_MOTIVORECHAZO"));
	    	return asentamiento;
	        
			}
		}
	
	private class RowMapperAsentamiento implements RowMapper{
		public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
			Asentamiento item = new Asentamiento();
			item.setCodDocumentoPendiente(rs.getString("STR_CODASENTAMIENTO"));
			item.setCodEstado(rs.getString("IND_CODESTADO"));
			item.setCodUOrganizativa(rs.getString("STR_CODUORGANIZATIVA"));
			item.setFechaAceptacionDocPte(rs.getDate("FEC_FECHAACEPTACION"));
			item.setFechaRechazoDocPte(rs.getDate("FEC_FECHARECHAZO"));
			item.setFechaRegistroDocPte(rs.getDate("FEC_FECHAREGISTRO"));
			return item;
		}
	}

	
	@SuppressWarnings("unchecked")
	public List<Asentamiento> consultaAsentamientosPorCriterios(Asentamiento asentamiento, Map<String, Map<String, TipoCondicion>> listaMetodos) {
		
		return this.jdbcTemplate.query("SELECT ASE.STR_CODASENTAMIENTO, ASE.STR_ASUNTO, ASE.FEC_FECHAREGISTRO, " +
				"ASE.FEC_FECHADOCUMENTO, ASE.FEC_FECHAPRESENTACION, ASE.FEC_FECHAACEPTACION, ASE.FEC_FECHARECHAZO, " +
				"ASE.STR_TIPOASENTAMIENTO, ASE.FEC_FECHAACEPTACION, ASE.FEC_FECHARECHAZO, ASE.STR_TIPOASENTAMIENTO, " +
				"ASE.STR_VIAPRESENTACION, ASE.STR_SOPORTEFISICO, ASE.IND_URGENTE, ASE.IND_FALTADOCESCANER, " +
				"ASE.STR_OBSERVACIONES, ASE.STR_CODUORGANIZATIVA, ASE.IND_CODESTADO, ASE.STR_CODEXPEDIENTE, " +
				"ASE.STR_CODINTERESADO, ASE.STR_MOTIVORECHAZO, EXPE.STR_CODEXPEDIENTE, EXPE.STR_CODTRAMITE " +
				"FROM ADD_ASENTAMIENTO ASE INNER JOIN ADD_EXPEDIENTE EXPE ON ASE.STR_CODEXPEDIENTE = EXPE.STR_CODEXPEDIENTE" +
					AddJdbcUtils.obtieneCriteriosConCondicion(asentamiento, listaMetodos) + 
					" ORDER BY STR_CODASENTAMIENTO ", new AsentamientoMapper());		
	}
	
	@SuppressWarnings("unchecked")
	public List<Asentamiento> consultaAsentamientosPorCriteriosOr(Asentamiento asentamiento, Map<String, Map<String, TipoCondicion>> listaMetodos) {
		
		return this.jdbcTemplate.query("SELECT ASE.STR_CODASENTAMIENTO, ASE.STR_ASUNTO, ASE.FEC_FECHAREGISTRO, " +
				"ASE.FEC_FECHADOCUMENTO, ASE.FEC_FECHAPRESENTACION, ASE.FEC_FECHAACEPTACION, ASE.FEC_FECHARECHAZO, " +
				"ASE.STR_TIPOASENTAMIENTO, ASE.FEC_FECHAACEPTACION, ASE.FEC_FECHARECHAZO, ASE.STR_TIPOASENTAMIENTO, " +
				"ASE.STR_VIAPRESENTACION, ASE.STR_SOPORTEFISICO, ASE.IND_URGENTE, ASE.IND_FALTADOCESCANER, " +
				"ASE.STR_OBSERVACIONES, ASE.STR_CODUORGANIZATIVA, ASE.IND_CODESTADO, ASE.STR_CODEXPEDIENTE, " +
				"ASE.STR_CODINTERESADO, ASE.STR_MOTIVORECHAZO, EXPE.STR_CODEXPEDIENTE, EXPE.STR_CODTRAMITE " +
				"FROM ADD_ASENTAMIENTO ASE INNER JOIN ADD_EXPEDIENTE EXPE ON ASE.STR_CODEXPEDIENTE = EXPE.STR_CODEXPEDIENTE " +
				"WHERE ASE.IND_CODESTADO = '0'" +
					AddJdbcUtils.obtieneCriteriosConCondicionContinuarOr(asentamiento, listaMetodos) + 
					" ORDER BY STR_CODASENTAMIENTO", new AsentamientoMapper());		
	}
	
	private class AsentamientoMapper implements RowMapper {
		 public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
			
	    	Asentamiento asentamiento = new Asentamiento();
	    	asentamiento.setCodAsentamiento(rs.getString("STR_CODASENTAMIENTO"));
	    	asentamiento.setAsunto(rs.getString("STR_ASUNTO"));
	    	asentamiento.setFechaRegistro(rs.getDate("FEC_FECHAREGISTRO"));
	    	asentamiento.setFechaDocumento(rs.getDate("FEC_FECHADOCUMENTO"));
	    	asentamiento.setFechaPresentacion(rs.getDate("FEC_FECHAPRESENTACION"));
	    	asentamiento.setFechaAceptacion(rs.getDate("FEC_FECHAACEPTACION"));
	    	asentamiento.setFechaRechazo(rs.getDate("FEC_FECHARECHAZO"));
	    	asentamiento.setTipoAsentamiento(rs.getString("STR_TIPOASENTAMIENTO"));
	    	asentamiento.setViaPresentacion(rs.getString("STR_VIAPRESENTACION"));
	    	asentamiento.setSoporteFisico(rs.getString("STR_SOPORTEFISICO"));
	    	asentamiento.setUrgente(rs.getString("IND_URGENTE"));
	    	asentamiento.setFaltaDocEscaner(rs.getString("IND_FALTADOCESCANER"));
	    	asentamiento.setObservaciones(rs.getString("STR_OBSERVACIONES"));
	    	asentamiento.setCodUOrganizativa(rs.getString("STR_CODUORGANIZATIVA"));
	    	asentamiento.setCodEstado(rs.getString("IND_CODESTADO"));
	    	asentamiento.setCodExpediente(rs.getString("STR_CODEXPEDIENTE"));
	    	asentamiento.setCodInteresado(rs.getString("STR_CODINTERESADO"));
	    	asentamiento.setMotivoRechazo(rs.getString("STR_MOTIVORECHAZO"));
	    	asentamiento.setCodTramite(rs.getString("STR_CODTRAMITE"));
	        return asentamiento;
	        
			}
		}
	
	public Asentamiento consultaAsentamientoPorCodigo(String elegido) {
		
		return (Asentamiento) this.jdbcTemplate.queryForObject("SELECT * FROM ADD_ASENTAMIENTO WHERE STR_CODASENTAMIENTO=?" ,
														   new Object[] { elegido },
														   new RMapperAsentamiento ());	
	}
	
	private class RMapperAsentamiento implements RowMapper {
		 public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
			
	    	Asentamiento asentamiento = new Asentamiento();
	    	asentamiento.setCodAsentamiento(rs.getString("STR_CODASENTAMIENTO"));
	    	asentamiento.setAsunto(rs.getString("STR_ASUNTO"));
	    	asentamiento.setFechaRegistro(rs.getDate("FEC_FECHAREGISTRO"));
	    	asentamiento.setFechaDocumento(rs.getDate("FEC_FECHADOCUMENTO"));
	    	asentamiento.setFechaPresentacion(rs.getDate("FEC_FECHAPRESENTACION"));
	    	asentamiento.setFechaAceptacion(rs.getDate("FEC_FECHAACEPTACION"));
	    	asentamiento.setFechaRechazo(rs.getDate("FEC_FECHARECHAZO"));
	    	asentamiento.setTipoAsentamiento(rs.getString("STR_TIPOASENTAMIENTO"));
	    	asentamiento.setViaPresentacion(rs.getString("STR_VIAPRESENTACION"));
	    	asentamiento.setSoporteFisico(rs.getString("STR_SOPORTEFISICO"));
	    	asentamiento.setUrgente(rs.getString("IND_URGENTE"));
	    	asentamiento.setFaltaDocEscaner(rs.getString("IND_FALTADOCESCANER"));
	    	asentamiento.setObservaciones(rs.getString("STR_OBSERVACIONES"));
	    	asentamiento.setCodUOrganizativa(rs.getString("STR_CODUORGANIZATIVA"));
	    	asentamiento.setCodEstado(rs.getString("IND_CODESTADO"));
	    	asentamiento.setCodExpediente(rs.getString("STR_CODEXPEDIENTE"));
	    	asentamiento.setCodInteresado(rs.getString("STR_CODINTERESADO"));
	    	asentamiento.setMotivoRechazo(rs.getString("STR_MOTIVORECHAZO"));
	        return asentamiento;
	        
			}
		}
	
	public int rechazarAsentamiento(Asentamiento asentamiento) {
		return jdbcTemplate.update("UPDATE ADD_ASENTAMIENTO SET IND_RECHAZADO = 'S' , STR_MOTIVORECHAZO= ?" + 
				"WHERE STR_CODASENTAMIENTO = ?",
				new Object[]{
							asentamiento.getMotivoRechazo() + " ", // + asentamiento.getMotivoOtros(),
							asentamiento.getCodAsentamiento()
							});
	}

	public int guardarAsentamiento(Asentamiento asentamiento) {
		return jdbcTemplate.update("UPDATE ADD_ASENTAMIENTO SET STR_CODEXPEDIENTE = ? WHERE STR_CODASENTAMIENTO = ?",
				new Object[]{
							asentamiento.getCodExpediente(),
							asentamiento.getCodAsentamiento()
							});
	}
}
