package es.alten.diputacion.add.abstraccion.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import es.alten.diputacion.add.abstraccion.ComboDao;

import es.alten.diputacion.add.modelo.Combo;
import es.alten.diputacion.add.modelo.RegistroCombo;

public class ComboDaoImpl implements ComboDao{

	private JdbcTemplate jdbcTemplate;
	
	public List<Combo> consultarTodosCombos() {
		@SuppressWarnings("unchecked")
		List<Combo> listaCombos=this.jdbcTemplate.query(
			    "select distinct ADD_COMBOS.INT_ID_COMBO, ADD_COMBOS.STR_DES_COMBO from ADD_COMBOS",
			    new RowMapperCombo());
		return listaCombos;
	}

	public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
	}

	public JdbcTemplate getJdbcTemplate() {
		return jdbcTemplate;
	}
	
	private class RowMapperCombo implements RowMapper{
		public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
	        	Combo combo = new Combo();
	            combo.setIdCombo(rs.getInt("INT_ID_COMBO"));
	            combo.setDescCombo(rs.getString("STR_DES_COMBO"));
	            return combo;
		}
	}

	public List<RegistroCombo> consultarTodosRegistrosPorCombo(int idCombo) {
		@SuppressWarnings("unchecked")
		List<RegistroCombo> listaRegistros=this.jdbcTemplate.query("select distinct ADD_COMBOS.INT_ID_REGISTRO, ADD_COMBOS.STR_DES_REGISTRO, " +
				"ADD_COMBOS.STR_VALOR_REGISTRO from ADD_COMBOS where ADD_COMBOS.INT_ID_COMBO=?",
				new Object[] { idCombo },	new RowMapperRegistroCombo());
		return listaRegistros;
	}
	private class RowMapperRegistroCombo implements RowMapper{
		public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
	        	RegistroCombo registro = new RegistroCombo();
	        	registro.setIdRegistro(rs.getInt("INT_ID_REGISTRO"));
	        	registro.setDescRegistro(rs.getString("STR_DES_REGISTRO"));
	        	registro.setValorRegistro(rs.getString("STR_VALOR_REGISTRO"));
	            return registro;
		}
	}

	public void deleteRegistroPorCombo(int idCombo, int idRegistro) {
		this.jdbcTemplate.update("delete from ADD_COMBOS WHERE ADD_COMBOS.INT_ID_COMBO=? AND ADD_COMBOS.INT_ID_REGISTRO=?",new Object[]{idCombo, idRegistro});
		
	}

	public RegistroCombo consultarRegistroCombo(int idCombo, int idRegistro) {
		
		try {
			return (RegistroCombo) this.jdbcTemplate.queryForObject("select * from ADD_COMBOS WHERE ADD_COMBOS.INT_ID_COMBO=? AND ADD_COMBOS.INT_ID_REGISTRO=?",
					new Object[]{idCombo, idRegistro},
					new RowMapperRegistroCombo());
		} catch (EmptyResultDataAccessException e) {
			return null;
		}
	}

	public void updateRegistroPorCombo(int idCombo, int idRegistro, String descRegistro, String valorRegistro) {
		this.jdbcTemplate.update("update ADD_COMBOS set STR_DES_REGISTRO=?, STR_VALOR_REGISTRO=? WHERE ADD_COMBOS.INT_ID_COMBO=? AND ADD_COMBOS.INT_ID_REGISTRO=?",
				new Object[]{descRegistro, valorRegistro, idCombo, idRegistro});	
		
	}

	public void insertRegistroPorCombo(int idCombo, String descCombo,
			String descRegistro, String valorRegistro) {
		int codigoRegistro = this.jdbcTemplate.queryForInt("SELECT MAX(INT_ID_REGISTRO) FROM ADD_COMBOS");
		codigoRegistro = codigoRegistro+1;
		String sql ="insert into ADD_COMBOS(" +
		"INT_ID_COMBO," +
		"INT_ID_REGISTRO," +
		"STR_DES_COMBO," +
		"STR_DES_REGISTRO," +
		"STR_VALOR_REGISTRO) values(?,?,?,?,?)" ;
Object[] parametros = new Object[]{
		idCombo,
		codigoRegistro,
		descCombo,
		descRegistro,
		valorRegistro
};
jdbcTemplate.update(sql,parametros);
	}
}
