package es.alten.diputacion.add.abstraccion.jdbc;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import org.apache.log4j.Logger;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import es.alten.diputacion.add.abstraccion.DocEdibamDao;
import es.alten.diputacion.add.modelo.DocumentoEdibam;
import es.alten.diputacion.add.modelo.Ida;

public class DocEdibamDaoImpl implements DocEdibamDao{
	Logger log = Logger.getLogger(DocumentoEdibam.class);
	private JdbcTemplate jdbcTemplate;
	public int selectCountDocumentosPorEns(Long idEntidad){		
		String sql = "select count(*) from ADD_DOCUMENTOEDIBAM where int_identidad=?";
		Object parametros[] = new Object[]{idEntidad};
		return jdbcTemplate.queryForInt(sql,parametros);
	}
	
	public DocumentoEdibam insertDocumentoEdibamImpl(DocumentoEdibam doc){		
		if(doc.getIdObjeto()==null){
			long codigoObjeto;
			codigoObjeto =  this.jdbcTemplate.queryForInt("select max(INT_IDOBJETO) from add_documentoedibam");
			doc.setIdObjeto(codigoObjeto + 1);
		}

		String sql = "insert into ADD_DOCUMENTOEDIBAM (" +
				"INT_IDENTIDAD," +
				"INT_IDCLASE," +
				"INT_IDOBJETO," +
				"STR_TITULO," +
				"STR_DESCRIPCION," +
				"STR_BORRADO," +
				"STR_TIPO)" +
				" values(?,?,?,?,?,?,?)";
		Object[] parametros = new Object[]{
				doc.getIdIdentidad(),
				doc.getIdClase(),
				doc.getIdObjeto(),
				doc.getTitulo(),
				doc.getDescripcion(),
				doc.getBorrado(),
				doc.getTipo()
		};
		jdbcTemplate.update(sql,parametros);
		return doc;
	}
	
	public void updateDocumentoEdibam(DocumentoEdibam doc){
		String sql="update ADD_DOCUMENTOEDIBAM set " +
		"INT_IDENTIDAD=?," +
		"INT_IDCLASE=?," +
		"INT_IDOBJETO=?," +
		"STR_TITULO=?," +
		"STR_DESCRIPCION=?," +
		"STR_BORRADO=?," +
		"STR_TIPO=?" +
		" where INT_IDENTIDAD=? and INT_IDCLASE=? and INT_IDOBJETO=?";
		Object[] parametros = new Object[]{
				doc.getIdIdentidad(),
				doc.getIdClase(),
				doc.getIdObjeto(),
				doc.getTitulo(),
				doc.getDescripcion(),
				doc.getBorrado(),
				doc.getTipo(),
				doc.getIdIdentidad(),
				doc.getIdClase(),
				doc.getIdObjeto()
		};
		jdbcTemplate.update(sql, parametros);
	}
	
	public int deleteDocumentoEdibam(Long idIdentidad,Long idClase, Long idObjeto) {
		String sql = "delete from ADD_DOCUMENTOEDIBAM where INT_IDENTIDAD=? and INT_IDCLASE=? and INT_IDOBJETO=? and STR_BORRADO='"+ DocumentoEdibam.NO+"'";
		Object[] parametros = new Object[]{
				idIdentidad,
				idClase,
				idObjeto,
		};
		return jdbcTemplate.update(sql, parametros);
	}
	
	public DocumentoEdibam selectDocumentoEdibam(Long idEntidad,Long idObjecte, Long idClasse) {
		String sql="select * from ADD_DOCUMENTOEDIBAM " +
		"where INT_IDENTIDAD=? and INT_IDCLASE=? and INT_IDOBJETO=?";
		Object[] parametros = new Object[]{new Long(idEntidad),new Long(idClasse),new Long(idObjecte)};
		return (DocumentoEdibam) this.jdbcTemplate.queryForObject(sql,parametros, new RowMapperDocumentoEdibam());
	}
	
	public int selectCountDocumentoEdibamPorTitulo(String titulo) {
		String sql="select count(*) from ADD_DOCUMENTOEDIBAM " +
		"where STR_TITULO=?";
		Object[] parametros = new Object[]{new String(titulo)};
		return this.jdbcTemplate.queryForInt(sql, parametros);
	}

	private class RowMapperDocumentoEdibam implements RowMapper{
		public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
			DocumentoEdibam doc = new DocumentoEdibam();
			doc.setIdClase(rs.getLong("INT_IDCLASE"));
			doc.setIdIdentidad(rs.getLong("INT_IDENTIDAD"));
			doc.setIdObjeto(rs.getLong("INT_IDOBJETO"));
			doc.setTitulo(rs.getString("STR_TITULO"));
			doc.setDescripcion("STR_DESCRIPCION");
			doc.setBorrado(rs.getString("STR_BORRADO"));
			doc.setTipo(rs.getString("STR_TIPO"));
			return doc;
		}
	}

	public JdbcTemplate getJdbcTemplate() {
		return jdbcTemplate;
	}

	public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
	}
/*
	@SuppressWarnings("unchecked")
	public List<DocumentoEdibam> selectDocumentoEdibamPorMultiplesBusquedas(String titulo,String descripcion, Map<String, String> mapTagOr,Map<String, String> mapTagsAnd){
		String sql ="select ADD_DOCUMENTOEDIBAM.int_identidad, ADD_DOCUMENTOEDIBAM.int_idclase, ADD_DOCUMENTOEDIBAM.int_idobjeto" +
				" from ADD_DOCUMENTOEDIBAM, ADD_TAG where ADD_DOCUMENTOEDIBAM.INT_IDCLASE = ADD_TAG.INT_IDCLASE and  ADD_DOCUMENTOEDIBAM.INT_IDOBJETO = ADD_TAG.INT_IDOBJETO and ADD_DOCUMENTOEDIBAM.INT_IDENTIDAD = ADD_TAG.INT_IDENTIDAD and" +
				" (UPPER(ADD_DOCUMENTOEDIBAM.STR_DESCRIPCION) LIKE UPPER('%?%') or ADD_DOCUMENTOEDIBAM.STR_DESCRIPCION=null)" +
				" and (UPPER(ADD_DOCUMENTOEDIBAM.STR_TITULO) LIKE UPPER('%?%') or ADD_DOCUMENTOEDIBAM.STR_TITULO=null) " +
				" group by ADD_DOCUMENTOEDIBAM.int_identidad, ADD_DOCUMENTOEDIBAM.int_idclase, ADD_DOCUMENTOEDIBAM.int_idobjeto";
		@SuppressWarnings("rawtypes")
		List listaParametros = new ArrayList();
		listaParametros.add(titulo);
		listaParametros.add(descripcion);
		
		if(mapTagOr.entrySet().size()>0 || mapTagsAnd.entrySet().size()>0){
			sql = sql + "and (";
			boolean primeraVezTagsValor=true;
			for(Entry<String, String>tagOr:mapTagOr.entrySet()){
				if(primeraVezTagsValor==false){
					sql = sql + " or ";
				}
				sql = sql + "(STR_NOMBRE_TAG=? and STR_VALOR=?)";
				listaParametros.add(tagOr.getKey());
				listaParametros.add(tagOr.getValue());
				primeraVezTagsValor=false;
			}
			if(primeraVezTagsValor==false){
				if( mapTagsAnd.entrySet().size()>0){
					sql = sql + " or (";
					sql = sql + getSqlTagsAnd(mapTagsAnd, listaParametros);
					sql = sql + ")";
				}
			}
			else {
				sql = sql + getSqlTagsAnd(mapTagsAnd, listaParametros);
			}
			sql = sql + ")";
		}
		
		try {
			return jdbcTemplate.queryForList(sql, listaParametros.toArray());
		}
		catch(EmptyResultDataAccessException e){
			return new ArrayList<DocumentoEdibam>();
		}
	}
	*/
	private String getSubConsulta(String campoASeleccionar, String titulo,String descripcion, Map<String, String> mapTagOr,Map<String, String> mapTagsAnd){
		String sql ="select * from ADD_DOCUMENTOEDIBAM, ADD_TAG where ADD_DOCUMENTOEDIBAM.INT_IDCLASE = ADD_TAG.INT_IDCLASE and  ADD_DOCUMENTOEDIBAM.INT_IDOBJETO = ADD_TAG.INT_IDOBJETO and ADD_DOCUMENTOEDIBAM.INT_IDENTIDAD = ADD_TAG.INT_IDENTIDAD and" +
		" (UPPER(ADD_DOCUMENTOEDIBAM.STR_DESCRIPCION) LIKE UPPER('%?%') or ADD_DOCUMENTOEDIBAM.STR_DESCRIPCION=null)" +
		" and (UPPER(ADD_DOCUMENTOEDIBAM.STR_TITULO) LIKE UPPER('%?%') or ADD_DOCUMENTOEDIBAM.STR_TITULO=null)";
			@SuppressWarnings("rawtypes")
			List listaParametros = new ArrayList();
			listaParametros.add(titulo);
			listaParametros.add(descripcion);
			
			if(mapTagOr.entrySet().size()>0 || mapTagsAnd.entrySet().size()>0){
				sql = sql + "and (";
				boolean primeraVezTagsValor=true;
				for(Entry<String, String>tagOr:mapTagOr.entrySet()){
					if(primeraVezTagsValor==false){
						sql = sql + " or ";
					}
					sql = sql + "(STR_NOMBRE_TAG=? and STR_VALOR=?)";
					listaParametros.add(tagOr.getKey());
					listaParametros.add(tagOr.getValue());
					primeraVezTagsValor=false;
				}
				if(primeraVezTagsValor==false){
					if( mapTagsAnd.entrySet().size()>0){
						sql = sql + " or (";
						sql = sql + getSqlTagsAnd(mapTagsAnd, listaParametros);
						sql = sql + ")";
					}
				}
				else {
					sql = sql + getSqlTagsAnd(mapTagsAnd, listaParametros);
				}
				sql = sql + ")";
			}
			return sql;

	}
	
	
	private String getSqlTagsAnd(Map<String, String> mapTags, List<Object>listaParametros){
		String sqlTagsAnd ="";
		boolean primeraVezTagsAnd=true;
		for(Entry<String, String>tagAnd:mapTags.entrySet()){
			if(primeraVezTagsAnd==false){
				sqlTagsAnd = sqlTagsAnd + " and ";
			}
			sqlTagsAnd = sqlTagsAnd + "(STR_NOMBRE_TAG=? and STR_VALOR=?)";
			listaParametros.add(tagAnd.getKey());
			listaParametros.add(tagAnd.getValue());
			primeraVezTagsAnd=false;
		}
		return sqlTagsAnd;
	}
	
	private String getSqlTags(Map<String, String> mapTags, List<Object>listaParametros){
		String sqlTagsAnd ="";
		boolean primeraVezTagsAnd=true;
		for(Entry<String, String>tagAnd:mapTags.entrySet()){
			if(primeraVezTagsAnd==false){
				sqlTagsAnd = sqlTagsAnd + " or ";
			}
			sqlTagsAnd = sqlTagsAnd + "(t.STR_NOMBRE_TAG=? and t.STR_VALOR=?)";
			listaParametros.add(tagAnd.getKey());
			listaParametros.add(tagAnd.getValue());
			primeraVezTagsAnd=false;
		}
		if(primeraVezTagsAnd==true){
			return " 1=1 ";
		}
		return sqlTagsAnd;
	}
	
	private String getSqlIdClass(List<Long>listaIdClass, List<Object>listaParametros){
		String sqlTagsAnd ="";
		boolean primeraVezTagsAnd=true;
		
		if(listaIdClass.size()>0){
			sqlTagsAnd = sqlTagsAnd + " and (";
		}
		
		for(Long idClass:listaIdClass){
			if(primeraVezTagsAnd==false){
				sqlTagsAnd = sqlTagsAnd + " or ";
			}
			
			sqlTagsAnd = sqlTagsAnd + "d.INT_IDCLASE=?";
			listaParametros.add(idClass);
			primeraVezTagsAnd=false;
		}
		
		if(listaIdClass.size()>0){
			sqlTagsAnd = sqlTagsAnd + ") ";
		}
		return sqlTagsAnd;
	}

	public List<Ida> selectIda(List<Long> listaIdClass,Long idIdentidad, Map<String, String> tags) {
		String sql = "select d.int_identidad, d.int_idclase, d.int_idobjeto" +
		" from addbd.add_documentoedibam d, addbd.add_tag t " +
		"where d.INT_IDENTIDAD = t.INT_IDENTIDAD and d.INT_IDCLASE = t.INT_IDCLASE and " +
		"d.INT_IDOBJETO = t.INT_IDOBJETO and d.INT_IDENTIDAD=?"; 
		List<Object> listaParametros = new ArrayList<Object>();
		listaParametros.add(idIdentidad);
		sql = sql + getSqlIdClass(listaIdClass, listaParametros) + " and " +getSqlTags(tags, listaParametros);
		sql = sql + " group by d.int_identidad, d.int_idclase, d.int_idobjeto";
		List listaDocumentosEdibam;
		try {
			listaDocumentosEdibam =  jdbcTemplate.query(sql,listaParametros.toArray(),new RowMapperIda());
		}
		catch(EmptyResultDataAccessException e){
			listaDocumentosEdibam = new ArrayList<DocumentoEdibam>();
		}
		return listaDocumentosEdibam;
	}
	
	private class RowMapperIda implements RowMapper{
		public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
			Ida ida = new Ida();
			ida.setIdClase(rs.getLong("INT_IDCLASE"));
			ida.setIdIdentidad(rs.getLong("INT_IDENTIDAD"));
			ida.setIdObjeto(rs.getLong("INT_IDOBJETO"));
			return ida;
		}
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public List<Ida> selectIdaPorMultiplesBusquedas(String titulo,String descripcion, Map<String, String> mapTagOr,Map<String, String> mapTagsAnd) {
		String sql ="select ADD_DOCUMENTOEDIBAM.int_identidad, ADD_DOCUMENTOEDIBAM.int_idclase, ADD_DOCUMENTOEDIBAM.int_idobjeto" +
		" from ADD_DOCUMENTOEDIBAM, ADD_TAG where ADD_DOCUMENTOEDIBAM.INT_IDCLASE = ADD_TAG.INT_IDCLASE and  ADD_DOCUMENTOEDIBAM.INT_IDOBJETO = ADD_TAG.INT_IDOBJETO and ADD_DOCUMENTOEDIBAM.INT_IDENTIDAD = ADD_TAG.INT_IDENTIDAD and" +
		" (UPPER(ADD_DOCUMENTOEDIBAM.STR_DESCRIPCION) LIKE UPPER(?) or ?=null)" +
		" and (UPPER(ADD_DOCUMENTOEDIBAM.STR_TITULO) LIKE UPPER(?) or ?=null) ";

		List listaParametros = new ArrayList();
		listaParametros.add("%" + descripcion + "%");
		listaParametros.add(descripcion);
		listaParametros.add("%" + titulo + "%");
		listaParametros.add(titulo);
		if(mapTagOr.entrySet().size()>0 || mapTagsAnd.entrySet().size()>0){
			sql = sql + " and (";
			boolean primeraVezTagsValor=true;
			for(Entry<String, String>tagOr:mapTagOr.entrySet()){
				if(primeraVezTagsValor==false){
					sql = sql + " or ";
				}
				sql = sql + "(STR_NOMBRE_TAG=? and STR_VALOR=?)";
				listaParametros.add(tagOr.getKey());
				listaParametros.add(tagOr.getValue());
				primeraVezTagsValor=false;
			}
			if(primeraVezTagsValor==false){
				if( mapTagsAnd.entrySet().size()>0){
					sql = sql + " or (";
					sql = sql + getSqlTagsAnd(mapTagsAnd, listaParametros);
					sql = sql + ")";
				}
			}
			else {
				sql = sql + getSqlTagsAnd(mapTagsAnd, listaParametros);
			}
			sql = sql + ")";
		}
		sql = sql + " group by ADD_DOCUMENTOEDIBAM.int_identidad, ADD_DOCUMENTOEDIBAM.int_idclase, ADD_DOCUMENTOEDIBAM.int_idobjeto";
		try {
			System.out.println(sql);
			return jdbcTemplate.query(sql, listaParametros.toArray(),new RowMapperIda());
		}
		catch(EmptyResultDataAccessException e){
			return new ArrayList<Ida>();
		}
	}
}
