package es.alten.diputacion.add.abstraccion.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import es.alten.diputacion.add.abstraccion.DocumentoDao;
import es.alten.diputacion.add.modelo.Documento;
import es.alten.diputacion.add.util.AddJdbcUtils;
import es.alten.diputacion.add.util.SecuenciadorProvicional;
import es.alten.diputacion.add.util.TipoCondicion;

public class DocumentoDaoImpl implements DocumentoDao{
	
	private JdbcTemplate jdbcTemplate;
	
	public JdbcTemplate getJdbcTemplate() {
		return jdbcTemplate;
	}

	public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
	}

	@SuppressWarnings("unchecked")
	public List<Documento> consultaDocumentosPorCriterios(Documento documento, Map<String, Map<String, TipoCondicion>> listaMetodos) {
		
		return this.jdbcTemplate.query("SELECT * FROM ADD_DOCUMENTO" + 
					AddJdbcUtils.obtieneCriteriosConCondicion(documento, listaMetodos) + 
					" ORDER BY STR_CODDOCUMENTO ", new RowMapperDocumento());		
	}
	
	@SuppressWarnings("unchecked")
	public List<Documento> consultaDocumentosPorCriteriosOr(Documento documento, Map<String, Map<String, TipoCondicion>> listaMetodos) {
		
		return this.jdbcTemplate.query("SELECT * FROM ADD_DOCUMENTO" + 
					AddJdbcUtils.obtieneCriteriosConCondicionOr(documento, listaMetodos) + 
					" ORDER BY STR_CODDOCUMENTO ", new RowMapperDocumento());		
	}
	
	/*private class DocumentoMapper implements RowMapper {
		public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
	    	Documento documento = new Documento();
	    	documento.setCodDocumento(rs.getString("STR_CODDOCUMENTO"));
	       	documento.setCodExpediente(rs.getString("STR_CODEXPEDIENTE"));
	       	documento.setSoporte(rs.getString("STR_SOPORTE"));
	       	documento.setNivelDescripcion(rs.getString("STR_NIVELDESCRIPCION"));
	    	documento.setTituloDocumento(rs.getString("STR_TITULODOCUMENTO"));
	    	documento.setProductor(rs.getString("STR_PRODUCTOR"));
	    	documento.setUnidadProductora(rs.getString("STR_UNIDADPRODUCTORA"));
	    	documento.setFechaDocumento(rs.getDate("FEC_FECHACADUCIDAD"));
	    	documento.setFechaDocumento(rs.getDate("FEC_FECHADOCUMENTO"));
	    	documento.setCodInteresado(rs.getString("STR_CODINTERESADO"));
	    	documento.setTipoDoc(rs.getString("STR_TIPODOC"));
	    	
	        return documento;   
		}
	}*/
	
	public Documento consultaDocumentoPorCodigo(String elegido) {
		return (Documento) this.jdbcTemplate.queryForObject("SELECT * FROM ADD_DOCUMENTO WHERE STR_CODDOCUMENTO=?",
				new Object[] { elegido }, new RowMapperDocumento ());	
	}
	
	private class RowMapperDocumento implements RowMapper {
		public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
			Documento documento = new Documento();
	    	documento.setCodDocumento(rs.getString("STR_CODDOCUMENTO"));
	       	documento.setCodExpediente(rs.getString("STR_CODEXPEDIENTE"));
	       	documento.setSoporte(rs.getString("STR_SOPORTE"));
	       	documento.setNivelDescripcion(rs.getString("STR_NIVELDESCRIPCION"));
	    	documento.setTituloDocumento(rs.getString("STR_TITULODOCUMENTO"));
	    	documento.setProductor(rs.getString("STR_PRODUCTOR"));
	    	documento.setUnidadProductora(rs.getString("STR_UNIDADPRODUCTORA"));
	    	documento.setFechaCaducidad(rs.getDate("FEC_FECHACADUCIDAD"));
	    	documento.setFechaDocumento(rs.getDate("FEC_FECHADOCUMENTO"));
	    	documento.setCodInteresado(rs.getString("STR_CODINTERESADO"));
	    	documento.setCodAsentamiento(rs.getString("STR_CODASENTAMIENTO"));
	    	documento.setVisible(rs.getString("IND_VISIBLE"));
	        return documento;
	    }
	}

	@SuppressWarnings("unchecked")
	public List<Documento> consultaDocumentosPorCodigoExpediente(
			String codExpediente) {
		return this.jdbcTemplate.query("SELECT * FROM ADD_DOCUMENTO " + 
				"WHERE STR_CODEXPEDIENTE = ? AND IND_VISIBLE='S'", new Object []{codExpediente},new RowMapperDocumento());
	}

	public void deleteDocumento(String codDocumento) {
		try {
			this.jdbcTemplate.update("UPDATE ADD_DOCUMENTO SET IND_VISIBLE='N', FEC_FECHACADUCIDAD=SYSDATE() WHERE STR_CODDOCUMENTO=?",new Object[]{codDocumento});
		} catch (DataAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	public int insertDocumento(Documento documento) {
		try {
			//int codigoDocumento = this.jdbcTemplate.queryForInt("SELECT MAX(STR_CODDOCUMENTO) FROM ADD_DOCUMENTO");
			//codigoDocumento = codigoDocumento+1;
			int codigoDocumento = SecuenciadorProvicional.getSecuencia();
			String codDocumento = new Integer(codigoDocumento).toString();
			String sql="INSERT INTO ADD_DOCUMENTO (STR_CODDOCUMENTO, STR_CODEXPEDIENTE, STR_SOPORTE, STR_NIVELDESCRIPCION, " +
					"STR_TITULODOCUMENTO, STR_PRODUCTOR, STR_UNIDADPRODUCTORA,FEC_FECHACADUCIDAD,FEC_FECHADOCUMENTO,STR_CODINTERESADO,STR_CODASENTAMIENTO,IND_VISIBLE) VALUES (?,?,?,?,?,?,?,?,?,?,?,?)";
			Object[] parametros = new Object[]{
					codDocumento,
					documento.getCodExpediente(),
					documento.getSoporte(),
					documento.getNivelDescripcion(),
					documento.getTituloDocumento(),
					documento.getProductor(),
					documento.getUnidadProductora(),
					documento.getFechaCaducidad(),
					documento.getFechaDocumento(),
					documento.getCodInteresado(),
					documento.getCodAsentamiento(),
					documento.getVisible()};
			jdbcTemplate.update(sql,parametros);
			return codigoDocumento;
		} catch (DataAccessException e) {
			e.printStackTrace();
			return 0;
		}
	}
}
