package es.alten.diputacion.add.abstraccion.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import es.alten.diputacion.add.abstraccion.ExpedienteDao;
import es.alten.diputacion.add.modelo.Expediente;
import es.alten.diputacion.add.modelo.ExpedienteExpandido;
import es.alten.diputacion.add.modelo.Persona;
import es.alten.diputacion.add.util.AddJdbcUtils;
import es.alten.diputacion.add.util.SecuenciadorProvicional;
import es.alten.diputacion.add.util.TipoCondicion;

public class ExpedienteDaoImpl implements ExpedienteDao{
	
	private JdbcTemplate jdbcTemplate;
	
	public JdbcTemplate getJdbcTemplate() {
		return jdbcTemplate;
	}

	public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
	}

	public void insertarExpediente(Expediente expediente) {
		String sql ="insert into ADD_EXPEDIENTE(" +
				"STR_CODEXPEDIENTE," +
				"STR_CODUORGANIZATIVA," +
				"STR_CODTRANSITOEXPEDIENTE," +
				"FEC_FECHAREGISTROEXPEDIENTE," +
				"FEC_FECHABAJAEXPEDIENTE," +
				"IND_CODESTADO," +
				"STR_CODSUBCATEGORIA," +
				"STR_CODCATEGORIA," +
				"STR_CODTRAMITE) values(?,?,?,?,?,?,?,?,?)" ;
		Object[] parametros = new Object[]{
				expediente.getCodExpediente(),
				expediente.getCodUorganizativa(),
				expediente.getCodTransitoExp(),
				expediente.getFechaRegistroExp(),
				expediente.getFechaBajaExp(),
				expediente.getCodEstado(),
				expediente.getCodSubcategoria(),
				expediente.getCodCategoria(),
				expediente.getCodTramite()
		};
		jdbcTemplate.update(sql,parametros);
	}

	public void updateExpediente(Expediente expediente) {
		String sql ="update ADD_EXPEDIENTE set " +
				"STR_CODUORGANIZATIVA=?," +
				"STR_CODTRANSITOEXPEDIENTE=?," +
				"FEC_FECHAREGISTROEXPEDIENTE=?," +
				"FEC_FECHABAJAEXPEDIENTE=?," +
				"IND_CODESTADO=?," +
				"STR_CODSUBCATEGORIA=?," +
				"STR_CODCATEGORIA=?," +
				"STR_CODTRAMITE=?" +
				" where STR_CODEXPEDIENTE=?";
		
		Object[] parametros = new Object[]{
				expediente.getCodUorganizativa(),
				expediente.getCodTransitoExp(),
				expediente.getFechaRegistroExp(),
				expediente.getFechaBajaExp(),
				expediente.getCodEstado(),
				expediente.getCodSubcategoria(),
				expediente.getCodCategoria(),
				expediente.getCodTramite(),
				expediente.getCodExpediente()
		};
		jdbcTemplate.update(sql, parametros);		
	}

	@SuppressWarnings("unchecked")
	public List<Expediente> consultarTodasExpedientesAbiertos() {		
		List<Expediente> exp_abiertos = this.jdbcTemplate.query("select * from ADD_EXPEDIENTE WHERE IND_CODESTADO = '3' AND IND_VISIBLE = 'S'",
			    new RowMapper() {
			public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
	        	Expediente expAbierto = new Expediente();
	        	expAbierto.setCodCategoria(rs.getString("STR_CODCATEGORIA"));
	        	expAbierto.setCodEstado(rs.getString("IND_CODESTADO"));
	        	expAbierto.setCodExpediente(rs.getString("STR_CODEXPEDIENTE"));
	        	expAbierto.setCodSubcategoria(rs.getString("STR_CODSUBCATEGORIA"));
	        	expAbierto.setCodTransitoExp(rs.getString("STR_CODTRANSITOEXPEDIENTE"));
	        	expAbierto.setCodUorganizativa(rs.getString("STR_CODUORGANIZATIVA"));
	        	expAbierto.setFechaBajaExp(rs.getDate("FEC_FECHABAJAEXPEDIENTE"));
	        	expAbierto.setFechaRegistroExp(rs.getDate("FEC_FECHAREGISTROEXPEDIENTE"));
	        	expAbierto.setCodTramite(rs.getString("STR_CODTRAMITE"));
	            return expAbierto;
			}
		});
		return exp_abiertos;
	}

	public List<Expediente> consultarTodasExpedientesCerrados() {
		@SuppressWarnings("unchecked")
		List<Expediente> exp_cerrados = this.jdbcTemplate.query("select * from ADD_EXPEDIENTE where IND_CODESTADO = '4' AND IND_VISIBLE = 'S'",
			    new RowMapper() {
	        public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
	        	Expediente expCerrado = new Expediente();

	        	expCerrado.setCodCategoria(rs.getString("STR_CODCATEGORIA"));
	        	expCerrado.setCodEstado(rs.getString("IND_CODESTADO"));
	        	expCerrado.setCodExpediente(rs.getString("STR_CODEXPEDIENTE"));
	        	expCerrado.setCodSubcategoria(rs.getString("STR_CODSUBCATEGORIA"));
	        	expCerrado.setCodTransitoExp(rs.getString("STR_CODTRANSITOEXPEDIENTE"));
	        	expCerrado.setCodUorganizativa(rs.getString("STR_CODUORGANIZATIVA"));
	        	expCerrado.setFechaBajaExp(rs.getDate("FEC_FECHABAJAEXPEDIENTE"));
	        	expCerrado.setFechaRegistroExp(rs.getDate("FEC_FECHAREGISTROEXPEDIENTE"));

	            return expCerrado;
	        }
	    });
		return exp_cerrados;
	}

	public List<Expediente> consultarTodasExpedientesRechazados() {
		@SuppressWarnings("unchecked")
		List<Expediente> exp_rechazados = this.jdbcTemplate.query(
			    "select * from ADD_EXPEDIENTE where IND_CODESTADO = '1' AND IND_VISIBLE = 'S'",
			    new RowMapper() {
	        public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
	        	Expediente expRechazado = new Expediente();
	        	
	        	expRechazado.setCodCategoria(rs.getString("STR_CODCATEGORIA"));
	        	expRechazado.setCodEstado(rs.getString("IND_CODESTADO"));
	        	expRechazado.setCodExpediente(rs.getString("STR_CODEXPEDIENTE"));
	        	expRechazado.setCodSubcategoria(rs.getString("STR_CODSUBCATEGORIA"));
	        	expRechazado.setCodTransitoExp(rs.getString("STR_CODTRANSITOEXPEDIENTE"));
	        	expRechazado.setCodUorganizativa(rs.getString("STR_CODUORGANIZATIVA"));
	        	expRechazado.setFechaBajaExp(rs.getDate("FEC_FECHABAJAEXPEDIENTE"));
	        	expRechazado.setFechaRegistroExp(rs.getDate("FEC_FECHAREGISTROEXPEDIENTE"));
	        	
	            return expRechazado;
	        }
	    });
		return exp_rechazados;
	}
	
	@SuppressWarnings("unchecked")
	public List<ExpedienteExpandido> consultaExpedientesPorCriterios(ExpedienteExpandido expedienteExpandido, Map<String, Map<String, TipoCondicion>> listaMetodos) {
		return this.jdbcTemplate.query("SELECT * FROM " +
					"ADD_EXPEDIENTE EXPE INNER JOIN ADD_PERSONA PERS ON EXPE.STR_CODINTERESADO = PERS.STR_CODPERSONA" +
					AddJdbcUtils.obtieneCriteriosConCondicion(expedienteExpandido, listaMetodos) + 
					" ORDER BY EXPE.STR_CODEXPEDIENTE ", new RowMapperExpedienteExpandido());		
	}
	
	@SuppressWarnings("unchecked")
	public List<ExpedienteExpandido> consultaExpedientesPorCriteriosOr(ExpedienteExpandido expedienteExpandido, Map<String, Map<String, TipoCondicion>> listaMetodos) {
		return this.jdbcTemplate.query("SELECT * FROM " +
					"ADD_EXPEDIENTE EXPE INNER JOIN ADD_PERSONA PERS ON EXPE.STR_CODINTERESADO = PERS.STR_CODPERSONA" +
					AddJdbcUtils.obtieneCriteriosConCondicionOr(expedienteExpandido, listaMetodos) + 
					" ORDER BY EXPE.STR_CODEXPEDIENTE ", new RowMapperExpedienteExpandido());		
	}
	
	
	
	private class ExpedientePersonaMapper implements RowMapper {
		 public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
	    	Expediente expediente = new Expediente();
	    	Persona persona = new Persona();
	    	expediente.setCodEstado(rs.getString("IND_CODESTADO"));
	    	expediente.setCodExpediente(rs.getString("STR_CODEXPEDIENTE"));
	    	expediente.setDescripcion(rs.getString("STR_DESCRIPCION"));
	    	expediente.setCodInteresado(rs.getString("STR_CODINTERESADO"));
	    	expediente.setFechaRegistroExp(rs.getDate("FEC_FECHAINICIOEXPEDIENTE"));
	    	persona.setCodigo(rs.getString("STR_CODPERSONA"));
	    	persona.setNombre(rs.getString("STR_NOMBREPERSONA"));
	    	persona.setApellido1 (rs.getString("STR_APELLIDO1PERSONA"));
	    	persona.setApellido2 (rs.getString("STR_APELLIDO2PERSONA"));
	    	persona.setNumDocumento (rs.getString("STR_NUMDOCUMENTOPERSONA"));
	    	expediente.setInteresado(persona);
	        return expediente;    
		}
	}

	public Expediente consultaExpedientePorCodigo(String elegido) {
		return (Expediente) this.jdbcTemplate.queryForObject("SELECT * FROM ADD_EXPEDIENTE WHERE STR_CODEXPEDIENTE=? AND IND_VISIBLE = 'S'" ,
														   new Object[] { elegido }, new RowMapperExpediente ());	
	}
	
	private class RowMapperExpediente implements RowMapper {
		 public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
	    	Expediente expediente = new Expediente();
	    	expediente.setCodExpediente(rs.getString("STR_CODEXPEDIENTE"));
	    	expediente.setCodUorganizativa(rs.getString("STR_CODUORGANIZATIVA"));
	    	expediente.setCodTransitoExp(rs.getString("STR_CODTRANSITOEXPEDIENTE"));
	    	expediente.setFechaRegistroExp(rs.getDate("FEC_FECHAINICIOEXPEDIENTE"));
	    	expediente.setFechaBajaExp(rs.getDate("FEC_FECHAFINALEXPEDIENTE"));        	
	    	expediente.setCodEstado(rs.getString("IND_CODESTADO"));
	    	expediente.setCodCategoria(rs.getString("STR_CODTIPOLOGIA"));
	    	expediente.setCodSubcategoria(rs.getString("STR_CODSUBTIPOLOGIA"));
	    	expediente.setTipoExpediente(rs.getString("STR_TIPOEXPEDIENTE"));
	    	expediente.setNivelDescripcion(rs.getString("INT_NIVELDESCRIPCION"));
	    	expediente.setTituloExpediente(rs.getString("INT_TITULOEXPEDIENTE"));
	    	expediente.setPoliticaEvaluacion(rs.getString("STR_POLITICAEVALUACION"));
	    	expediente.setDescripcion(rs.getString("STR_DESCRIPCION"));	 
	    	expediente.setObservaciones(rs.getString("STR_OBSERVACIONES"));
	    	expediente.setPublicidad(rs.getString("IND_PUBLICIDAD"));
	    	expediente.setCodTramite(rs.getString("STR_CODTRAMITE"));	    	
	    	expediente.setCodUsuario(rs.getString("STR_CODUSUARIO"));
	    	expediente.setNivelProtecDatos(rs.getString("STR_NIVELPROTECDATOSPERSONALES"));
	    	expediente.setEstadoTramitacion(rs.getString("STR_CODESTADOTRAMITACION"));
	    	expediente.setCodInteresado(rs.getString("STR_CODINTERESADO"));
	    	expediente.setCodRepresentante(rs.getString("STR_CODREPRESENTANTE"));
	        return expediente;
		}
	}

	@SuppressWarnings("unchecked")
	public List<Expediente> consultaExpedientesPorCriteriosAsentamiento(Expediente expediente, Map<String, Map<String, TipoCondicion>> listaMetodos) {
		return this.jdbcTemplate.query("SELECT EX.STR_CODEXPEDIENTE, EX.IND_CODESTADO, EX.STR_DESCRIPCION, EX.IND_CODESTADO, " +
				"EX.STR_CODINTERESADO, EX.FEC_FECHAINICIOEXPEDIENTE, PE.STR_CODPERSONA, PE.STR_NOMBREPERSONA, " +
				"EX.STR_CODTRAMITE, PE.STR_APELLIDO1PERSONA, PE.STR_APELLIDO2PERSONA, PE.STR_NUMDOCUMENTOPERSONA FROM " +
				"ADD_EXPEDIENTE EX INNER JOIN ADD_PERSONA PE ON EX.STR_CODINTERESADO = PE.STR_CODPERSONA" +
				AddJdbcUtils.obtieneCriteriosConCondicion(expediente, listaMetodos) + 
				" ORDER BY EX.STR_CODEXPEDIENTE ", new ExpedientePersonaMapper());
	}

	@SuppressWarnings("unchecked")
	public List<Expediente> consultarTodosExpedientes() {
		return this.jdbcTemplate.query("SELECT EXPE.STR_CODEXPEDIENTE, EXPE.IND_CODESTADO, EXPE.STR_DESCRIPCION, " +
				"EXPE.STR_CODINTERESADO, EXPE.FEC_FECHAINICIOEXPEDIENTE, PERS.STR_CODPERSONA, PERS.STR_NOMBREPERSONA, " +
				"PERS.STR_APELLIDO1PERSONA, PERS.STR_APELLIDO2PERSONA, PERS.STR_NUMDOCUMENTOPERSONA FROM " +
				"ADD_EXPEDIENTE EXPE INNER JOIN ADD_PERSONA PERS ON EXPE.STR_CODINTERESADO = PERS.STR_CODPERSONA" +
				" ORDER BY EXPE.STR_CODEXPEDIENTE WHERE IND_VISIBLE = 'S'", new ExpedientePersonaMapper());
	}

	public ExpedienteExpandido consultaExpedienteCodigo(String codExpediente) {
		return (ExpedienteExpandido) this.jdbcTemplate.queryForObject("SELECT * FROM ADD_EXPEDIENTE WHERE STR_CODEXPEDIENTE=? AND IND_VISIBLE = 'S'" ,
				   new Object[] { codExpediente }, new RowMapperExpedienteExpandido ());
	}
	
	private class RowMapperExpedienteExpandido implements RowMapper {
		 public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
			ExpedienteExpandido expediente = new ExpedienteExpandido();
	    	expediente.setCodExpediente(rs.getString("STR_CODEXPEDIENTE"));
	    	expediente.setCodUorganizativa(rs.getString("STR_CODUORGANIZATIVA"));
	    	expediente.setCodTransitoExp(rs.getString("STR_CODTRANSITOEXPEDIENTE"));
	    	expediente.setFechaRegistroExp(rs.getDate("FEC_FECHAINICIOEXPEDIENTE"));
	    	expediente.setFechaBajaExp(rs.getDate("FEC_FECHAFINALEXPEDIENTE"));        	
	    	expediente.setCodEstado(rs.getString("IND_CODESTADO"));
	    	expediente.setCodCategoria(rs.getString("STR_CODTIPOLOGIA"));
	    	expediente.setCodSubcategoria(rs.getString("STR_CODSUBTIPOLOGIA"));
	    	expediente.setTipoExpediente(rs.getString("STR_TIPOEXPEDIENTE"));
	    	expediente.setNivelDescripcion(rs.getString("INT_NIVELDESCRIPCION"));
	    	expediente.setTituloExpediente(rs.getString("INT_TITULOEXPEDIENTE"));
	    	expediente.setPoliticaEvaluacion(rs.getString("STR_POLITICAEVALUACION"));
	    	expediente.setDescripcion(rs.getString("STR_DESCRIPCION"));	 
	    	expediente.setObservaciones(rs.getString("STR_OBSERVACIONES"));
	    	expediente.setPublicidad(rs.getString("IND_PUBLICIDAD"));
	    	expediente.setCodTramite(rs.getString("STR_CODTRAMITE"));	    	
	    	expediente.setCodUsuario(rs.getString("STR_CODUSUARIO"));
	    	expediente.setNivelProtecDatos(rs.getString("STR_NIVELPROTECDATOSPERSONALES"));
	    	expediente.setEstadoTramitacion(rs.getString("STR_CODESTADOTRAMITACION"));
	    	expediente.setCodInteresado(rs.getString("STR_CODINTERESADO"));
	    	expediente.setCodRepresentante(rs.getString("STR_CODREPRESENTANTE"));
	        return expediente;
		}
	}

	public int updateExpedienteExpandido(ExpedienteExpandido expedienteExpandido) {
		try {
			return this.jdbcTemplate.update("UPDATE ADD_EXPEDIENTE SET FEC_FECHAINICIOEXPEDIENTE=?, " + 
					"FEC_FECHAFINALEXPEDIENTE=?, IND_CODESTADO=?, STR_CODUORGANIZATIVA=?, IND_PUBLICIDAD=?, STR_CODUSUARIO=?, " + 
					"STR_NIVELPROTECDATOSPERSONALES=?, INT_TITULOEXPEDIENTE=?, STR_TIPOEXPEDIENTE=?, STR_DESCRIPCION=?, " + 
					"STR_OBSERVACIONES=? WHERE STR_CODEXPEDIENTE=?",
					new Object[]{
					expedienteExpandido.getFechaRegistroExp(),
					expedienteExpandido.getFechaBajaExp(),
					expedienteExpandido.getCodEstado(),
					expedienteExpandido.getCodUorganizativa(),
					expedienteExpandido.getPublicidad(),
					expedienteExpandido.getCodUsuario(),
					expedienteExpandido.getNivelProtecDatos(),
					expedienteExpandido.getTituloExpediente(),
					expedienteExpandido.getTipoExpediente(),
					expedienteExpandido.getDescripcion(), 
					expedienteExpandido.getObservaciones(), 
					expedienteExpandido.getCodExpediente()});
		} catch (DataAccessException e) {
			e.printStackTrace();
			return 0;
		}

	}

	public int eliminarExpediente(String elegido) {
		try {
			return this.jdbcTemplate.update("UPDATE ADD_EXPEDIENTE SET IND_VISIBLE = 'N' WHERE STR_CODEXPEDIENTE=?",
					new Object[]{elegido});
		} catch (DataAccessException e) {
			e.printStackTrace();
			return 0;
		}
}

	public int createExpediente(ExpedienteExpandido expedienteExpandido) {
		try {
			//int codigoExpediente = this.jdbcTemplate.queryForInt("SELECT MAX(STR_CODEXPEDIENTE) FROM ADD_EXPEDIENTE");
			//codigoExpediente = codigoExpediente+1;
			//String codExpediente = new Integer(codigoExpediente).toString();
			int codigoExpediente = SecuenciadorProvicional.getSecuencia();
			String codExpediente = new Integer(codigoExpediente).toString();
			this.jdbcTemplate.update("INSERT INTO ADD_EXPEDIENTE (STR_CODEXPEDIENTE, STR_CODUORGANIZATIVA, " +
					"STR_CODTRANSITOEXPEDIENTE, FEC_FECHAINICIOEXPEDIENTE, FEC_FECHAFINALEXPEDIENTE, IND_CODESTADO, " +
					"STR_CODTIPOLOGIA, STR_CODSUBTIPOLOGIA , STR_TIPOEXPEDIENTE, INT_NIVELDESCRIPCION, INT_TITULOEXPEDIENTE, " +
					"STR_POLITICAEVALUACION , STR_DESCRIPCION, STR_OBSERVACIONES,  IND_PUBLICIDAD, STR_CODTRAMITE, "+
					"STR_CODUSUARIO, STR_NIVELPROTECDATOSPERSONALES, STR_CODESTADOTRAMITACION, STR_CODINTERESADO, " +
					"STR_CODREPRESENTANTE, IND_VISIBLE) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
					new Object[]{
					codExpediente,
					expedienteExpandido.getCodUorganizativa(),
					expedienteExpandido.getCodTransitoExp(),
					expedienteExpandido.getFechaRegistroExp(),
					expedienteExpandido.getFechaBajaExp(),
					expedienteExpandido.getCodEstado(),					
					expedienteExpandido.getCodCategoria(),
					expedienteExpandido.getCodSubcategoria(),
					expedienteExpandido.getTipoExpediente(),
					expedienteExpandido.getNivelDescripcion(),
					expedienteExpandido.getTituloExpediente(),
					expedienteExpandido.getPoliticaEvaluacion(),
					expedienteExpandido.getDescripcion(),
					expedienteExpandido.getObservaciones(),
					expedienteExpandido.getPublicidad(),					
					expedienteExpandido.getCodTramite(),
					expedienteExpandido.getCodUsuario(),
					expedienteExpandido.getNivelProtecDatos(),
					expedienteExpandido.getEstadoTramitacion(),
					expedienteExpandido.getCodInteresado(),
					expedienteExpandido.getCodRepresentante(),
					expedienteExpandido.getVisible()});
			return codigoExpediente;
		} catch (DataAccessException e) {
			e.printStackTrace();
			return 0;
		}
	}
	
}
