package es.alten.diputacion.add.abstraccion.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import es.alten.diputacion.add.abstraccion.HistoricoTramitacionDao;
import es.alten.diputacion.add.modelo.HistoricoTramitacion;

public class HistoricoTramitacionDaoImpl implements HistoricoTramitacionDao {

	private JdbcTemplate jdbcTemplate;

	public JdbcTemplate getJdbcTemplate() {
		return jdbcTemplate;
	}

	public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
	}

	public List<HistoricoTramitacion> consultarHistoricoTramitacion(
			String codExpediente) {

		@SuppressWarnings("unchecked")
		List<HistoricoTramitacion> listaHistoricos = this.jdbcTemplate.query(
				"SELECT * FROM ADD_HISTORICOESTADOTRAMITACION "
						+ "WHERE STR_CODEXPEDIENTE = ?",
				new Object[] { codExpediente },
				new RowMapperHistoricoTramitacion());
		return listaHistoricos;
	}

	private class RowMapperHistoricoTramitacion implements RowMapper {
		public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
			HistoricoTramitacion historicoTramitacion = new HistoricoTramitacion();
			historicoTramitacion.setCodHistoricoTramitacion((rs
					.getString("COD_HISTORICOESTADOTRAMITACION")));
			historicoTramitacion.setFechaRegistro(rs
					.getDate("FEC_FECHAREGISTRO"));
			historicoTramitacion.setCodEstadoTramitacion(rs
					.getString("STR_CODESTADOTRAMITACION"));
			return historicoTramitacion;
		}
	}

	public int updateTramitacion(HistoricoTramitacion historicoTramitacion) {
		try {
			return this.jdbcTemplate
					.update("UPDATE ADD_HISTORICODESTADOTRAMITACION SET COD_ESTADOTRAMITACION=?, "
							+ "FEC_FECHAREGISTRO=?, WHERE COD_HISTORICOESTADOTRAMITACION=? AND STR_CODEXPEDIENTE=?",
							new Object[] {
									historicoTramitacion
											.getCodEstadoTramitacion(),
									historicoTramitacion.getFechaRegistro(),
									historicoTramitacion
											.getCodHistoricoTramitacion(),
									historicoTramitacion.getCodExpediente() });
		} catch (DataAccessException e) {
			e.printStackTrace();
			return 0;
		}
	}

	public int createHistoricoTramitacion(HistoricoTramitacion historicoTramitacion) {
		try {
			return this.jdbcTemplate.update("INSERT INTO ADD_HISTORICOESTADOTRAMITACION (COD_HISTORICOESTADOTRAMITACION, " +
					"FEC_FECHAREGISTRO ,STR_CODEXPEDIENTE, STR_CODESTADOTRAMITACION) VALUES (?,?,?,?)",
					new Object[]{
					historicoTramitacion.getCodHistoricoTramitacion(),
					historicoTramitacion.getFechaRegistro(),
					historicoTramitacion.getCodExpediente(),					
					historicoTramitacion.getCodEstadoTramitacion()});
		} catch (DataAccessException e) {
			e.printStackTrace();
			return 0;
		}
	}

}
