package es.alten.diputacion.add.abstraccion.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import es.alten.diputacion.add.abstraccion.OperacionProgramaDAO;
import es.alten.diputacion.add.modelo.OperacionPrograma;

/**
 * 
 */
public class OperacionProgramaDaoImpl implements OperacionProgramaDAO {

	private JdbcTemplate jdbcTemplate;

	public OperacionProgramaDaoImpl(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	public OperacionPrograma getOperacionProgramaPorCriterio(OperacionPrograma operacionPrograma) {
		String sql = "select * from OPERACION_PROGRAMA where upper(ACCION) = ?";
		Object[] parametros = new Object[]{operacionPrograma.getNombreAccion().toUpperCase()};
				
		if(operacionPrograma.getNombreOperacion() != null){
			 sql = "select * from OPERACION_PROGRAMA where upper(ACCION) = ? and upper(OPERACION) = ?";
			 parametros = new Object[]{
					operacionPrograma.getNombreAccion().toUpperCase(),
					operacionPrograma.getNombreOperacion().toUpperCase()};
		}
				
		OperacionPrograma busqueda = (OperacionPrograma) this.jdbcTemplate.queryForObject(sql, parametros, new RowMapper() {
			public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
				OperacionPrograma item = new OperacionPrograma();
				item.setCodigo(rs.getLong("CODIGO"));
				item.setDescripcion(rs.getString("DESCRIPCION"));
				item.setNombreAccion(rs.getString("ACCION"));
				item.setNombreOperacion(rs.getString("OPERACION"));
				return item;
			}
		});
		return busqueda;
	}

	public List<OperacionPrograma> getOperacionProgramaPorRol() {
		// TODO Auto-generated method stub
		return null;
	}

	public void insertarOperacionPrograma(OperacionPrograma operacionPrograma) {
		String sql = "insert into OPERACION_PROGRAMA(ACCION,OPERACION) values(?,?)";
		jdbcTemplate.update(sql, new Object[]{operacionPrograma.getNombreAccion(), operacionPrograma.getNombreOperacion()});
	}

	public Boolean isOperacionProgramaPorRol(long codigoRol, OperacionPrograma operacionPrograma) {
		try {
			OperacionPrograma encontrado = getOperacionProgramaPorCriterio(operacionPrograma);
			String sql = "select count(1) FROM PERFILES_OPERACIONES WHERE CODIGO_PERFIL = ? and CODIGO_OPERACION = ?";
			int numero = jdbcTemplate.queryForInt(sql,
			new Object[] { codigoRol, encontrado.getCodigo() });
			return (numero == 0) ? Boolean.FALSE : Boolean.TRUE;
		} catch (EmptyResultDataAccessException e) {
			return Boolean.FALSE;
		}
	}
	
}
