package es.alten.diputacion.add.abstraccion.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.log4j.Logger;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import es.alten.diputacion.add.abstraccion.PersonaDao;
import es.alten.diputacion.add.modelo.Persona;
import es.alten.diputacion.add.util.SecuenciadorProvicional;

public class PersonaDaoImpl implements PersonaDao{
	
	Logger log = Logger.getLogger(PersonaDaoImpl.class);
	private JdbcTemplate jdbcTemplate;	
	
	public JdbcTemplate getJdbcTemplate() {
		return jdbcTemplate;
	}

	public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
	}

	public Persona consultaPersonaByCodigo(String codInteresado) {
		try {
			StringBuffer sbQuery = new StringBuffer();
			sbQuery.append("SELECT * FROM ADD_PERSONA WHERE STR_CODPERSONA = ?");
			
			return (Persona) this.jdbcTemplate.queryForObject(sbQuery.toString(), 
					new Object[] { codInteresado }, new RowMapperPersona());
		} catch (EmptyResultDataAccessException e) {
			return null;
		}
	}
	
	private class RowMapperPersona implements RowMapper {
		public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
	    	Persona interesado = new Persona();
	    	interesado.setCodigo(rs.getString("STR_CODPERSONA"));
	    	interesado.setTipo(rs.getString("STR_TIPOPERSONA"));
	    	interesado.setNumDocumento(rs.getString("STR_NUMDOCUMENTOPERSONA"));
	    	interesado.setNombre(rs.getString("STR_NOMBREPERSONA"));
	    	interesado.setApellido1(rs.getString("STR_APELLIDO1PERSONA"));
	    	interesado.setApellido2(rs.getString("STR_APELLIDO2PERSONA"));
	    	interesado.setPoblacion(rs.getBigDecimal("INT_POBLACIONPERSONA"));
	    	interesado.setEmail(rs.getString("STR_EMAILPERSONA"));
	    	interesado.setTelefono(rs.getString("STR_TELEFONOPERSONA"));
	    	interesado.setDireccion(rs.getString("STR_DIRECCIONPERSONA"));
	        return interesado;
		}
	}

	public int updatePersona(Persona persona) {
		try {
			return this.jdbcTemplate.update("UPDATE ADD_PERSONA SET STR_NOMBREPERSONA=?, STR_APELLIDO1PERSONA=?, " + 
					"STR_APELLIDO2PERSONA=?, STR_EMAILPERSONA=?, STR_TELEFONOPERSONA=?, STR_DIRECCIONPERSONA=?, " + 
					"STR_NUMDOCUMENTOPERSONA=? WHERE STR_CODPERSONA=?",
					new Object[]{
					persona.getNombre(),
					persona.getApellido1(),
					persona.getApellido2(),
					persona.getEmail(),
					persona.getTelefono(),
					persona.getDireccion(),
					persona.getNumDocumento(),
					persona.getCodigo()});
		} catch (DataAccessException e) {
			e.printStackTrace();
			return 0;
		}
	}

	public int createPersona(Persona persona) {
		try {
			//int codigoPersona = this.jdbcTemplate.queryForInt("SELECT MAX(STR_CODPERSONA) FROM ADD_PERSONA");
			//codigoPersona = codigoPersona+1;
			//String codPersona = new Integer(codigoPersona).toString();
			//return
			int codigoPersona = SecuenciadorProvicional.getSecuencia();
			String codPersona = new Integer(codigoPersona).toString();
			this.jdbcTemplate.update("INSERT INTO ADD_PERSONA (STR_CODPERSONA, STR_TIPOPERSONA, STR_NUMDOCUMENTOPERSONA, " +
					"STR_NOMBREPERSONA, STR_APELLIDO1PERSONA, STR_APELLIDO2PERSONA, INT_POBLACIONPERSONA, STR_EMAILPERSONA, " +
					"STR_TELEFONOPERSONA, STR_DIRECCIONPERSONA) VALUES (?,?,?,?,?,?,?,?,?,?)",
					new Object[]{
					codPersona,
					persona.getTipo(),
					persona.getNumDocumento(),					
					persona.getNombre(),
					persona.getApellido1(),
					persona.getApellido2(),
					persona.getPoblacion(),
					persona.getEmail(),
					persona.getTelefono(),
					persona.getDireccion()});
			return codigoPersona;
		} catch (DataAccessException e) {
			e.printStackTrace();
			return 0;
		}
	}

}
