package es.alten.diputacion.add.abstraccion.jdbc;

import java.io.Serializable;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import org.apache.log4j.Logger;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import es.alten.diputacion.add.abstraccion.TagDao;
import es.alten.diputacion.add.modelo.Tag;


public class TagDaoImpl implements TagDao,Serializable{
	private Logger log = Logger.getLogger(TagDaoImpl.class);
	private JdbcTemplate jdbcTemplate;
	
	
	
	
	
	@SuppressWarnings("unchecked")
	public List<Tag> selectTagDao(Long idEntidad) {
		String sql = "select * from ADD_TAG where int_identidad=?";
		Object[] parametros = new Object[]{idEntidad};
		try {
			List<Tag> listaTag = jdbcTemplate.query(sql, parametros,new RowMapperTag());
			return listaTag;
		}
		catch(EmptyResultDataAccessException e){
			return new ArrayList<Tag>();
		}
	}
	
	public List<Tag> selectTagByIda(Long idEntidad,Long idClasse, Long idObjeto){
		String sql ="select * from add_tag where INT_IDENTIDAD=? and INT_IDCLASE=? and INT_IDOBJETO=?";
		Object[] parametros = new Object[]{idEntidad,idClasse,idObjeto};
		try {
			List<Tag> listaTag = jdbcTemplate.query(sql, parametros,new RowMapperTag());
			return listaTag;
		}
		catch(EmptyResultDataAccessException e){
			return new ArrayList<Tag>();
		}
	}

	private class RowMapperTag implements RowMapper{
		public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
			Tag tag = new Tag();
			tag.setNombre(rs.getString("STR_NOMBRE_TAG"));
			tag.setObligatorio(rs.getString("STR_OBLIGATORIO"));
			tag.setIdObjeto(rs.getLong("INT_IDOBJETO"));
			tag.setIdClase(rs.getLong("INT_IDCLASE"));
			tag.setIdIdentidad(rs.getLong("INT_IDENTIDAD"));
			tag.setValor(rs.getString("STR_VALOR"));
			return tag;
		}
	}

	public JdbcTemplate getJdbcTemplate() {
		return jdbcTemplate;
	}

	public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
	}

	public Tag insertTag(Tag tag) {
			String sql = "insert into ADD_TAG (" +
					"INT_IDENTIDAD," +
					"INT_IDCLASE," +
					"INT_IDOBJETO," +
					"STR_NOMBRE_TAG," +
					"STR_OBLIGATORIO, " +
					"STR_VALOR) values(?,?,?,?,?,?)";
			Object[] parametros = new Object[]{
					tag.getIdIdentidad(),
					tag.getIdClase(),
					tag.getIdObjeto(),
					tag.getNombre(),
					tag.getObligatorio(),
					tag.getValor()
			};
			jdbcTemplate.update(sql,parametros);
		return tag;
	}
}
