package es.alten.diputacion.add.abstraccion.jdbc;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import es.alten.diputacion.add.abstraccion.TramiteDao;
import es.alten.diputacion.add.modelo.Tramite;
import es.alten.diputacion.add.modelo.TramiteExpandido;
import es.alten.diputacion.add.util.AddJdbcUtils;
import es.alten.diputacion.add.util.SecuenciadorProvicional;
import es.alten.diputacion.add.util.TipoCondicion;

public class TramiteDaoImpl implements TramiteDao{
	
	Logger log = Logger.getLogger(TramiteDaoImpl.class);
	public static final String INSERT_SQL="INSERT INTO ADD_TRAMITE (STR_CODTRAMITE, STR_NOMBRETRAMITE, STR_DESCTRAMITE, " +
			"STR_CODSUBTIPOLOGIA, STR_TEMA, STR_FAMILIA, STR_SUBFAMILIA, STR_AREA, STR_ADMINISTRACIONCOMPONENTE, " +
			"STR_NIVELPROTECDATOSPERSONALES, STR_CLASIFICACIONARCHIVO, IND_PUBLICIDAD, STR_EVALSELECCIONELIMINACION, " +
			"STR_CODENTIDAD, STR_CODUORGANIZATIVADESTINO, FEC_FECHAREGISTRO, FEC_FECHABAJA, IND_ELIMINADO) " +
			"VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
	private JdbcTemplate jdbcTemplate;
	
	@SuppressWarnings("unchecked")
	public List<Tramite> consultarTramitesUsuario(String codUsuario) {
		return this.jdbcTemplate.query("SELECT * FROM ADD_TRAMITE, " + 
				"ADD_USUARIO_TRAMITE WHERE ADD_USUARIO_TRAMITE.STR_CODUSUARIO = '" + codUsuario + 
				"'AND ADD_USUARIO_TRAMITE.STR_CODTRAMITE=ADD_TRAMITE.STR_CODTRAMITE AND ADD_TRAMITE.IND_ELIMINADO='N'" ,
				new RowMapperTramite());
	}
	
	@SuppressWarnings("unchecked")
	public List<TramiteExpandido> consultaTramitesPorCriterios(TramiteExpandido tramiteExpandido, Map<String, Map<String, TipoCondicion>> listaMetodos) {
		return this.jdbcTemplate.query("SELECT * FROM ADD_TRAMITE" + 
					AddJdbcUtils.obtieneCriteriosConCondicion(tramiteExpandido, listaMetodos) + 
					" ORDER BY STR_CODTRAMITE ", new RowMapperTramiteExpandido());		
	}
	
	private class RowMapperTramiteExpandido implements RowMapper {
		public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
			
			TramiteExpandido tramiteExpandido = new TramiteExpandido();
			tramiteExpandido.setCodigoTramite(rs.getString("STR_CODTRAMITE"));
			tramiteExpandido.setNombreTramite(rs.getString("STR_NOMBRETRAMITE"));
			tramiteExpandido.setDescTramite(rs.getString("STR_DESCTRAMITE"));
			tramiteExpandido.setCodSubtipologia(rs.getString("STR_CODSUBTIPOLOGIA"));
			tramiteExpandido.setTema(rs.getString("STR_TEMA"));
			tramiteExpandido.setFamilia(rs.getString("STR_FAMILIA"));
			tramiteExpandido.setSubfamilia(rs.getString("STR_SUBFAMILIA"));
			tramiteExpandido.setArea(rs.getString("STR_AREA"));
			tramiteExpandido.setAdministracionComponente(rs.getString("STR_ADMINISTRACIONCOMPONENTE"));
	    	tramiteExpandido.setNivelProteccion(rs.getString("STR_NIVELPROTECDATOSPERSONALES"));
	    	tramiteExpandido.setPublicidad(rs.getString("IND_PUBLICIDAD"));
	    	tramiteExpandido.setEvalSeleccionEliminacion(rs.getString("STR_EVALSELECCIONELIMINACION"));
	    	tramiteExpandido.setCodEntidad(rs.getString("STR_CODENTIDAD"));

	    	tramiteExpandido.setCodUOrganizativaDestino(rs.getString("STR_CODUORGANIZATIVADESTINO"));
	    	tramiteExpandido.setClasificacionArchivo(rs.getString("STR_CLASIFICACIONARCHIVO"));
	
	    	tramiteExpandido.setFechaBaja(rs.getDate("FEC_FECHABAJA"));
	    	tramiteExpandido.setFechaRegistro(rs.getDate("FEC_FECHAREGISTRO"));
	    	tramiteExpandido.setEliminado(rs.getString("IND_ELIMINADO"));

	    	return tramiteExpandido;
		}
	}
	
	@SuppressWarnings("unchecked")
	public List<TramiteExpandido> consultaTramitesPorCriteriosOr(TramiteExpandido tramiteExpandido, Map<String, Map<String, TipoCondicion>> listaMetodos) {
		
		return this.jdbcTemplate.query("SELECT * FROM ADD_TRAMITE" + 
					AddJdbcUtils.obtieneCriteriosConCondicionOr(tramiteExpandido, listaMetodos) + 
					" ORDER BY STR_CODTRAMITE ", new RowMapperTramiteExpandido());		
	}

	public Tramite consultaTramitePorCodigo(String elegido) {
		return (Tramite) this.jdbcTemplate.queryForObject("SELECT * FROM ADD_TRAMITE WHERE STR_CODTRAMITE=? AND IND_ELIMINADO='N'" ,
														   new Object[] { elegido },
														   new RowMapperTramite ());	
	}

	@SuppressWarnings("unchecked")
	public List<Tramite> consultarTramitesTipologiaGestionable() {
		return this.jdbcTemplate.query("SELECT ADD_TRAMITE.* FROM ADD_TRAMITE, ADD_SUBTIPOLOGIA, ADD_TIPOLOGIA " +
				"WHERE ADD_TIPOLOGIA.STR_GESTIONABLE = 'S' AND ADD_TIPOLOGIA.STR_CODTIPOLOGIA=ADD_SUBTIPOLOGIA.STR_CODTIPOLOGIA " +
				"AND ADD_SUBTIPOLOGIA.STR_CODSUBTIPOLOGIA=ADD_TRAMITE.STR_CODSUBTIPOLOGIA AND ADD_TRAMITE.IND_ELIMINADO='N'" , 
				new RowMapperTramite());
	}
	
	private class RowMapperTramite implements RowMapper {
		public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
			Tramite tramite = new Tramite();
	    	tramite.setCodigoTramite(rs.getString("STR_CODTRAMITE"));
	    	tramite.setNombreTramite(rs.getString("STR_NOMBRETRAMITE"));
	    	tramite.setDescTramite(rs.getString("STR_DESCTRAMITE"));
	    	tramite.setCodSubtipologia(rs.getString("STR_CODSUBTIPOLOGIA"));
	    	tramite.setTema(rs.getString("STR_TEMA"));
	    	tramite.setFamilia(rs.getString("STR_FAMILIA"));
	    	tramite.setSubfamilia(rs.getString("STR_SUBFAMILIA"));
	    	tramite.setArea(rs.getString("STR_AREA"));
	    	tramite.setAdministracionComponente(rs.getString("STR_ADMINISTRACIONCOMPONENTE"));
	    	tramite.setNivelProteccion(rs.getString("STR_NIVELPROTECDATOSPERSONALES"));
	    	tramite.setPublicidad(rs.getString("IND_PUBLICIDAD"));
	    	tramite.setEvalSeleccionEliminacion(rs.getString("STR_EVALSELECCIONELIMINACION"));
	    	tramite.setCodEntidad(rs.getString("STR_CODENTIDAD"));

	    	tramite.setCodUOrganizativaDestino(rs.getString("STR_CODUORGANIZATIVADESTINO"));
	    	tramite.setClasificacionArchivo(rs.getString("STR_CLASIFICACIONARCHIVO"));
	
	    	tramite.setFechaBaja(rs.getDate("FEC_FECHABAJA"));
	    	tramite.setFechaRegistro(rs.getDate("FEC_FECHAREGISTRO"));
	    	tramite.setEliminado(rs.getString("IND_ELIMINADO"));

	    	return tramite;
		}
	}

	@SuppressWarnings("unchecked")
	public List<Tramite> consultarTodasTramites() {
			return this.jdbcTemplate.query( "SELECT * FROM ADD_TRAMITE " +
					"WHERE ADD_TRAMITE.IND_ELIMINADO = 'N' ORDER BY STR_NOMBRETRAMITE ASC",new RowMapperTramite());

	}
	
	public JdbcTemplate getJdbcTemplate() {
		return jdbcTemplate;
	}

	public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
	}

	public Tramite insertTramite(Tramite tramite) {
		try {
			//int codigoTramite = this.jdbcTemplate.queryForInt("SELECT MAX(STR_CODTRAMITE) FROM ADD_TRAMITE");
			//codigoTramite = codigoTramite+1;
			//String codTramite = new Integer(codigoTramite).toString();
			int codigoTramite = SecuenciadorProvicional.getSecuencia();
			String codTramite = new Integer(codigoTramite).toString();
			String sql="INSERT INTO ADD_TRAMITE (STR_CODTRAMITE, STR_NOMBRETRAMITE, STR_DESCTRAMITE, STR_CODSUBTIPOLOGIA, " +
					"STR_TEMA, STR_FAMILIA, STR_SUBFAMILIA, STR_AREA, STR_ADMINISTRACIONCOMPONENTE, STR_NIVELPROTECDATOSPERSONALES, " +
					"STR_CLASIFICACIONARCHIVO, IND_PUBLICIDAD, STR_EVALSELECCIONELIMINACION, STR_CODENTIDAD, STR_CODUORGANIZATIVADESTINO, " +
					"FEC_FECHAREGISTRO, FEC_FECHABAJA, IND_ELIMINADO) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
			Object[] parametros = new Object[]{
					codigoTramite,
					tramite.getNombreTramite(),
					tramite.getDescTramite(),
					tramite.getCodSubtipologia(),
					tramite.getTema(),
					tramite.getFamilia(),
					tramite.getSubfamilia(),
					tramite.getArea(),
					tramite.getAdministracionComponente(),
					tramite.getNivelProteccion(),
					tramite.getClasificacionArchivo(),
					tramite.getPublicidad(),
					tramite.getEvalSeleccionEliminacion(),
					tramite.getCodEntidad(),
					tramite.getCodUOrganizativaDestino(),
					tramite.getFechaRegistro(),
					tramite.getFechaBaja(),
					tramite.getEliminado()};
			jdbcTemplate.update(sql,parametros);
			return buscarTramitePorIdTramite(codTramite);
		} catch (DataAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
	}
	
	public Tramite buscarTramitePorIdTramite(String id_tramite) {
		try {
			return (Tramite)this.jdbcTemplate.queryForObject("SELECT * FROM ADD_TRAMITE " + 
				" WHERE STR_CODTRAMITE = ? AND ADD_TRAMITE.IND_ELIMINADO='N'", new Object[] { id_tramite }, new RowMapperTramite()); 
		} catch (EmptyResultDataAccessException e) { 
			return null; 
		}
	}
	
	@SuppressWarnings("unchecked")
	public List<Tramite> consultaTramitesByCodigoSubtipologia(String codSubtipologia) {
		return this.jdbcTemplate.query("SELECT * FROM ADD_TRAMITE " + 
				"WHERE STR_CODSUBTIPOLOGIA = ?" + 
				" ORDER BY STR_CODTRAMITE ", new Object []{codSubtipologia},new RowMapperTramite());
	}

	public Tramite updateTramite(Tramite tramite) {
		try {
			String sql="UPDATE ADD_TRAMITE SET " +
			"STR_CODTRAMITE=?," +
			"STR_NOMBRETRAMITE=?," +
			"STR_DESCTRAMITE=?," +
			"STR_CODSUBTIPOLOGIA=?," +
			"STR_TEMA=?," +
			"STR_FAMILIA=?," +
			"STR_SUBFAMILIA=?," +
			"STR_AREA=?," +
			"STR_ADMINISTRACIONCOMPONENTE=?," +
			"STR_NIVELPROTECDATOSPERSONALES=?," +
			"STR_CLASIFICACIONARCHIVO=?," +
			"IND_PUBLICIDAD=?," +
			"STR_EVALSELECCIONELIMINACION=?," +
			"STR_CODENTIDAD=?," +
			"STR_CODUORGANIZATIVADESTINO=?," +
			"FEC_FECHAREGISTRO=?," +
			"FEC_FECHABAJA=?," +
			"IND_ELIMINADO=?" +
			" WHERE STR_CODTRAMITE=?";
			
			Object[] parametros = new Object[]{
					tramite.getCodigoTramite(),
					tramite.getNombreTramite(),
					tramite.getDescTramite(),
					tramite.getCodSubtipologia(),
					tramite.getTema(),
					tramite.getFamilia(),
					tramite.getSubfamilia(),
					tramite.getArea(),
					tramite.getAdministracionComponente(),
					tramite.getNivelProteccion(),
					tramite.getClasificacionArchivo(),
					tramite.getPublicidad(),
					tramite.getEvalSeleccionEliminacion(),
					tramite.getCodEntidad(),
					tramite.getCodUOrganizativaDestino(),
					tramite.getFechaRegistro(),
					tramite.getFechaBaja(),
					tramite.getEliminado(),
					tramite.getCodigoTramite()};
			jdbcTemplate.update(sql,parametros);
		}
		catch(DataAccessException e){
			e.getMessage();
			e.getClass().getName();
		}
		String codTram=tramite.getCodigoTramite();
		
		return buscarTramitePorIdTramite(codTram);
	}
	
	public void deleteTramite(String idTramite) {
		
		try {
			this.jdbcTemplate.update("UPDATE ADD_TRAMITE SET IND_ELIMINADO='S', FEC_FECHABAJA=SYSDATE() WHERE STR_CODTRAMITE=?",new Object[]{idTramite});
		} catch (DataAccessException e) {
			e.printStackTrace();
		}
		
	}

}
