package es.alten.diputacion.add.abstraccion.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import es.alten.diputacion.add.abstraccion.UsuarioDAO;
import es.alten.diputacion.add.modelo.Usuario;
import es.alten.diputacion.add.util.SecuenciadorProvicional;


public class UsuarioDaoImpl implements UsuarioDAO {
	Logger log = Logger.getLogger(UsuarioDaoImpl.class);
	private JdbcTemplate jdbcTemplate;

	public JdbcTemplate getJdbcTemplate() {
		return jdbcTemplate;
	}

	public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
	}
	
	/*public Usuario consultarUsuarioPorUsuarioYEntidad(String codUsuario,String codigoEntidad) {
		try {
			String sql ="SELECT DISTINCT U.*  FROM ADD_USUARIO U,  ADD_USUARIO_UORGANIZATIVA UUO,  ADD_UORGANIZATIVA UO, ADD_ENTIDAD E " +
				"WHERE U.STR_CODUSUARIO = UUO.STR_CODUSUARIO  AND UUO.STR_CODUORGANIZATIVA = UO.STR_CODUORGANIZATIVA " +
				"AND UO.STR_CODENTIDAD = E.STR_CODENTIDAD AND U.STR_CODUSUARIO = ? AND E.STR_CODENTIDAD=?";
			return (Usuario) jdbcTemplate.queryForObject(sql, new Object[]{ codUsuario, codigoEntidad }, 
					new UsuarioDaoImpl.RowMapperUsuario());
		}
		catch(EmptyResultDataAccessException e){
			return null;
		}
	}*/
	
	public Usuario consultarUsuarioPorUsuarioYEntidad(String codUsuario) {
		Usuario usuario=(Usuario) this.jdbcTemplate.queryForObject(
			    "select * from ADD_USUARIO WHERE STR_CODUSUARIO=? AND ADD_USUARIO.IND_ELIMINADO = 'N'",
			    new Object[] { codUsuario }, 
			    new RowMapperUsuario());
		return usuario;	
	}	
		
	public int contarTodosUsuarios() {
		return this.jdbcTemplate.queryForInt("select count(0) from ADD_USUARIO");
	}

	private class RowMapperUsuario implements RowMapper{
		public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
	        	Usuario user = new Usuario();
	            user.setCodigo(rs.getString("STR_CODUSUARIO"));
	            user.setRol(rs.getString("IND_ROLUSUARIO"));
	            user.setNombre(rs.getString("STR_NOMBREUSUARIO"));
	            user.setPassword(rs.getString("STR_PASSWORD"));
	            user.setFechaAlta(rs.getDate("FEC_FECHAALTAUSUARIO"));
	            user.setFechaBaja(rs.getDate("FEC_FECHABAJAUSUARIO"));
	            user.setEliminado(rs.getString("IND_ELIMINADO"));
	            return user;
		}
	}

	@SuppressWarnings("unchecked")
	public List<Usuario> consultarTodosUsuarios() {
		List<Usuario> listaUsuario=this.jdbcTemplate.query(
			    "SELECT * FROM ADD_USUARIO WHERE ADD_USUARIO.IND_ELIMINADO = 'N' ORDER BY STR_NOMBREUSUARIO ASC", new RowMapperUsuario());
		return listaUsuario;
		
	}

	public Usuario insertUsuario(Usuario usuario) {
		try {
			//int codigoUsuario = this.jdbcTemplate.queryForInt("SELECT MAX(STR_CODUSUARIO) FROM ADD_USUARIO");
			//codigoUsuario = codigoUsuario+1;
			//String codUsuario = new Integer(codigoUsuario).toString();
			int codigoUsuario = SecuenciadorProvicional.getSecuencia();
			String codUsuario = new Integer(codigoUsuario).toString();
			String sql="INSERT INTO ADD_USUARIO (STR_CODUSUARIO, IND_ROLUSUARIO, STR_NOMBREUSUARIO, STR_PASSWORD, " +
					"FEC_FECHAALTAUSUARIO, FEC_FECHABAJAUSUARIO, IND_ELIMINADO) VALUES (?,?,?,?,?,?,?)";
			Object[] parametros = new Object[]{
					codigoUsuario,
					usuario.getRol(),
					usuario.getNombre(),
					usuario.getPassword(),
					usuario.getFechaAlta(),
					usuario.getFechaBaja(),
					usuario.getEliminado()};
			jdbcTemplate.update(sql,parametros);
			return buscarUsuarioPorIdUsuario(codUsuario);
		} catch (DataAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
	}

	public Usuario updateUsuario(Usuario usuario) {
		try {
			String sql="UPDATE ADD_USUARIO SET STR_CODUSUARIO=?, IND_ROLUSUARIO=?, STR_NOMBREUSUARIO=?, STR_PASSWORD=?, " +
					"FEC_FECHAALTAUSUARIO=?, FEC_FECHABAJAUSUARIO=?, IND_ELIMINADO=? WHERE STR_CODUSUARIO=?";
			Object[] parametros = new Object[]{
					usuario.getCodigo(),
					usuario.getRol(),
					usuario.getNombre(),
					usuario.getPassword(),
					usuario.getFechaAlta(),
					usuario.getFechaBaja(),
					usuario.getEliminado(),
					usuario.getCodigo()};
			jdbcTemplate.update(sql,parametros);
			
		} catch (DataAccessException e) {
			e.getMessage();
			e.getClass().getName();
		}
		String codUsuario=usuario.getCodigo();
		return buscarUsuarioPorIdUsuario(codUsuario);
	}

	public void deleteUsuario(String idUsuario) {
		try {
			this.jdbcTemplate.update("UPDATE ADD_USUARIO SET IND_ELIMINADO='S', FEC_FECHABAJAUSUARIO=SYSDATE() WHERE STR_CODUSUARIO=?",new Object[]{idUsuario});
		} catch (DataAccessException e) {
			e.printStackTrace();
		}
		
	}

	public Usuario buscarUsuarioPorIdUsuario(String id_usuario) {
		try {
			return (Usuario)this.jdbcTemplate.queryForObject("SELECT * FROM ADD_USUARIO " + 
				" WHERE STR_CODUSUARIO = ? AND ADD_USUARIO.IND_ELIMINADO='N'", new Object[] { id_usuario }, new RowMapperUsuario()); 
		} catch (EmptyResultDataAccessException e) { 
			return null; 
		}
	}	
}