/**
 * 
 */
package es.alten.diputacion.add.modelo;


/**
 * Enumerado que representa los roles del sistema de informacin hojas verdes.
 * 
 * @author Pedro
 * 
 */
public enum Roles {

	/**
	 * Puede ver todos los expedientes y realizar todas las operaciones. 
	 * Puede desbloquea registros y es el nico que puede dar de baja registros que 
	 * ya han sido gestionados e incluso pagados. Gestiona todas las 
	 * tablas del sistema y puede dar de alta al resto de los usuarios.
	 */
	AdministradorGeneral {

		public long getCodigo() {
			return 1;
		}

		public String getDescripcion() {
			return "AdministradorGeneral";
		}

	},
	/**
	 * Puede gestionar los usuarios de su unidad, y gestionar expedientes y pagos. 
	 * No puede borrar registros ni desbloquearlos. 
	 * Habr un administrador por cada unidad de las que se den de alta en la tabla de unidades. 
	 * No significa que sea el responsable de firma. 
	 * Puede dar de alta slo a los usuarios de su unidad.
	 */
	AdministradorUnidad {

		public long getCodigo() {
			return 2;
		}

		public String getDescripcion() {
			return "AdministradorUnidad";
		}

	},
	/**
	 * Pueden gestionar expedientes y pagos. 
	 * No puede borrar registros que hayan sido gestionados o tramitados pero si aquellos que no lo  hayan sido todava. 
	 * No puede dar de alta a otros usuarios. 
	 * Habr tantos usuarios por unidad como los  que se aprueben por la Secretara General y el resto de las Subdirecciones.
	 */
	UsuariosUnidad {
	
		public long getCodigo() {
			return 3;
		}

		public String getDescripcion() {
			return "UsuariosUnidad";
		}

	},
	/**
	 * Pueden gestionar slo pagos y darlos de baja si no han sido tramitados o gestionados. 
	 * Habr tantos por unidades segn las necesidades.
	 */
	TramitadoresPagos {

		public long getCodigo() {
			return 4;
		}

		public String getDescripcion() {
			return "TramitadoresPagos";
		}

	},
	/**
	 * Va a poder ver todos los expedientes y pagos de las unidades que tiene adscritas orgnicamente. 
	 * Los Subdirectores a su Subdireccin, los Jefes de Area a sus unidades, 
	 * pero no podrn operar sobre ellos: darlos de alta, baja, modificarlos, etc.. 
	 * Habr uno por Subdireccin y uno por cada rea.
	 */
	ConsultorUnidades {

		public long getCodigo() {
			return 5;
		}

		public String getDescripcion() {
			return "ConsultorUnidades";
		}

	},
	/**
	 * Va a poder ver todos los expedientes y pagos de todas las unidades del Organismo 
	 * (Director General, Secretario General, rea Econmica), 
	 * pero no podrn operar sobre ellos: darlos de alta, baja, modificarlos, etc. En principio habr tres.
	 */
	ConsultorGeneral {

		public long getCodigo() {
			return 6;
		}

		public String getDescripcion() {
			return "ConsultorGeneral";
		}

	},
	/**
	 * Va a poder ver todos los expedientes y pagos de todas las unidades pudiendo desbloquear 
	 * registros de pago pero no podr operar sobre ellos: darlos de alta, baja, modificarlos. 
	 * En principio habr dos.
	 */
	ConsultorDesbloqueador {

		public long getCodigo() {
			return 7;
		}

		public String getDescripcion() {
			return "ConsultorDesbloqueador";
		}

	};

	public abstract long getCodigo();

	public abstract String getDescripcion();

	/**
	 * Metodo que busca un Perfil a partir de su codigo.
	 * 
	 * @param buscado Codigo del Perfil buscado
	 * @return El Perfil buscado segun codigo
	 */
	public static Roles buscar(long buscado) {
		for (Roles actual : Roles.values())
			if (actual.getCodigo() == buscado)
				return actual;
		return null;
	}
	
	/**
	 * 
	 */
	public static Rol crearRol(long buscado) {
		Rol rol = new Rol();
		rol.setNumeroROL(buscado);
		rol.setDescripcion(buscar(buscado).getDescripcion());
		return rol;
	}

}
