package es.alten.diputacion.add.util;

import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

public class AddJdbcUtils {

	private static Logger LOGGER = Logger.getLogger(AddJdbcUtils.class);
	
	/**
	 * Funcin que te crea un criterio de bsqueda para una query
	 * @param objeto Objeto del que se extraen los datos para el criterio
	 * @param listaParametros listado de los parmetros del objeto que se quieren controlar
	 * @return Cadena con los criterios de la query
	 */
	
	public static String obtieneCriterios(Object objeto, Map<String, String> listaParametros){
		// Criterios que se devuelve con el where formado
		String criterios = "";
		// Mapa de criterios para formar el where
		List<String> listaCriterios = new ArrayList<String>();
		// Itera sobre todos los metodos que se quieren poner en el WHERE
		for (Map.Entry<String, String> entry: listaParametros.entrySet()) {
			String nombreMetodo = entry.getKey();
			String nombreColumna = entry.getValue();

			try {
				// Obtiene por reflexion el metodo del que saldra el valor
				Method metodo = objeto.getClass().getMethod(nombreMetodo);
				try {
					// Invoca el metodo para obtener el valor
					Object valor = metodo.invoke(objeto, new Object[0]);
					if(valor != null){
						String criterio = obtieneValorCriterio(nombreColumna, valor);
						if(!criterio.trim().equals("")){
							listaCriterios.add(criterio);
						}
					}
				} catch (Exception e) {
					LOGGER.error("Se ha producido un error al invocar " + nombreMetodo, e);
				}
			} catch (NoSuchMethodException e) {
				LOGGER.error("No se ha encontrado el metodo " + nombreMetodo, e);
			}
		}

		// Creamos el iterador para ir formando el where
		Iterator<String> iteradorCriterios = listaCriterios.iterator();
		int contador = 0;
		// Para cada criterio
		while(iteradorCriterios.hasNext()){
			// parmetro que se aade
			String criterio = (String)iteradorCriterios.next();
			if(contador == 0){
				criterios += " WHERE " + criterio;
			}else{
				criterios += " AND " + criterio;
			}
			contador++;
		}
		// Devolvemos los criterios del where para la consulta
		return criterios;
	}
	
	/**
	 * Funcion que te devuelve el criterio de un valor
	 * @param clave Tabla y campo de la que se crea el criterio
	 * @param valor Valor del campo que queremos comprobar
	 * @return El criterio completo de un campo
	 */
	private static String obtieneValorCriterio(String clave, Object valor){
		if(valor instanceof java.lang.String){
			// Comprobamos si la comparacin requiere un like
			if(!((String)valor).trim().equals("")){
				if(((String)valor).contains("*")){
					return clave + " like '" + ((String)valor).replaceAll("\\*", "%") + "'";
				}else{
					return clave + " = '" + (String)valor + "'";
				}
			}
			return "";
		}else if(valor instanceof Date){
			// Establecemos el formato que tendr la fecha
			SimpleDateFormat sdf = new SimpleDateFormat("MM/dd/yyyy");
			// TODO: Se tendr que permitir indicar el signo de comparacin
			return clave + " >= '" + sdf.format(valor) + " 00:00:00' AND " + clave + " <= '" + sdf.format(valor) + " 23:59:59'"; 
		}else if(valor instanceof Boolean){
			return ((Boolean)valor)? clave + " = 1" : clave + " = 0";
		}
		// En otro caso devolvemos el String del campo
		return clave + " = " + String.valueOf(valor);
	}
	
	/**
	 * Funcin que te crea un criterio de bsqueda para una query
	 * @param objeto Objeto del que se extraen los datos para el criterio
	 * @param listaParametros listado de los parmetros del objeto que se quieren controlar junto con la condicin
	 * 							que se quiere realizar
	 * @return Cadena con los criterios de la query
	 */
	public static String obtieneCriteriosConCondicion(Object objeto, Map<String, Map<String, TipoCondicion>> listaParametros){
		// Criterios que se devuelve con el where formado
		String criterios = "";
		// Mapa de criterios para formar el where
		List<String> listaCriterios = new ArrayList<String>();
		// Itera sobre todos los metodos que se quieren poner en el WHERE
		for (Map.Entry<String, Map<String, TipoCondicion>> entry: listaParametros.entrySet()) {
			String nombreMetodo = entry.getKey();
			Map<String, TipoCondicion> nombreColumna = entry.getValue();

			try {
				// Obtiene por reflexion el metodo del que saldra el valor
				Method metodo = objeto.getClass().getMethod(nombreMetodo);
				try {
					// Invoca el metodo para obtener el valor
					Object valor = metodo.invoke(objeto, new Object[0]);
					if(valor != null){
						String criterio = obtieneValorCriterioConCondicion(nombreColumna, valor);
						if(!criterio.trim().equals("")){
							listaCriterios.add(criterio);
						}
					}
				} catch (Exception e) {
					LOGGER.error("Se ha producido un error al invocar " + nombreMetodo, e);
				}
			} catch (NoSuchMethodException e) {
				LOGGER.error("No se ha encontrado el metodo " + nombreMetodo, e);
			}
		}

		// Creamos el iterador para ir formando el where
		Iterator<String> iteradorCriterios = listaCriterios.iterator();
		int contador = 0;
		// Para cada criterio
		while(iteradorCriterios.hasNext()){
			// parmetro que se aade
			String criterio = (String)iteradorCriterios.next();
			if(contador == 0){
				criterios += " WHERE " + criterio;
			}else{
				criterios += " AND " + criterio;
			}
			contador++;
		}
		// Devolvemos los criterios del where para la consulta
		return criterios;
	}
	
	/**
	 * Funcin que te crea un criterio de bsqueda para una query
	 * @param objeto Objeto del que se extraen los datos para el criterio
	 * @param listaParametros listado de los parmetros del objeto que se quieren controlar junto con la condicin
	 * 							que se quiere realizar
	 * @return Cadena con los criterios de la query
	 */
	public static String obtieneCriteriosConCondicionOr(Object objeto, Map<String, Map<String, TipoCondicion>> listaParametros){
		// Criterios que se devuelve con el where formado
		String criterios = "";
		// Mapa de criterios para formar el where
		List<String> listaCriterios = new ArrayList<String>();
		// Itera sobre todos los metodos que se quieren poner en el WHERE
		for (Map.Entry<String, Map<String, TipoCondicion>> entry: listaParametros.entrySet()) {
			String nombreMetodo = entry.getKey();
			Map<String, TipoCondicion> nombreColumna = entry.getValue();

			try {
				// Obtiene por reflexion el metodo del que saldra el valor
				Method metodo = objeto.getClass().getMethod(nombreMetodo);
				try {
					// Invoca el metodo para obtener el valor
					Object valor = metodo.invoke(objeto, new Object[0]);
					if(valor != null){
						String criterio = obtieneValorCriterioConCondicion(nombreColumna, valor);
						if(!criterio.trim().equals("")){
							listaCriterios.add(criterio);
						}
					}
				} catch (Exception e) {
					LOGGER.error("Se ha producido un error al invocar " + nombreMetodo, e);
				}
			} catch (NoSuchMethodException e) {
				LOGGER.error("No se ha encontrado el metodo " + nombreMetodo, e);
			}
		}

		// Creamos el iterador para ir formando el where
		Iterator<String> iteradorCriterios = listaCriterios.iterator();
		int contador = 0;
		// Para cada criterio
		while(iteradorCriterios.hasNext()){
			// parmetro que se aade
			String criterio = (String)iteradorCriterios.next();
			if(contador == 0){
				criterios += " WHERE " + criterio;
			}else{
				criterios += " OR " + criterio;
			}
			contador++;
		}
		// Devolvemos los criterios del where para la consulta
		return criterios;
	}
	
	/**
	 * Funcion que te devuelve el criterio de un valor
	 * @param clave Tabla y campo de la que se crea el criterio
	 * @param valor Valor del campo que queremos comprobar
	 * @return El criterio completo de un campo
	 */
	private static String obtieneValorCriterioConCondicion(Map<String, TipoCondicion> datos, Object valor){
		for (Map.Entry<String, TipoCondicion> entry: datos.entrySet()) {
			if(valor instanceof java.lang.String[]){
				if (entry.getValue() == TipoCondicion.In){
					return entry.getKey() + " " + entry.getValue().getOperador() + " " + construirCondicionIn(valor);	
				}else{
					return "";
				}				
			}else if(valor instanceof java.lang.String){
				// Comprobamos si la comparacin requiere un like
				if(!((String)valor).trim().equals("")){
					if (entry.getValue() == TipoCondicion.IgualParecido){
						if(((String)valor).contains("*")){
							return entry.getKey() + " like '" + ((String)valor).replaceAll("\\*", "%") + "'";
						}else{
							return entry.getKey() + " = '" + (String)valor + "'";
						}
					}else{
						return entry.getKey() + " " + entry.getValue().getOperador() + " '" + (String)valor + "'";
					}
				}
				return "";
			}else if(valor instanceof Date){
				// Establecemos el formato que tendr la fecha
				SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
				// TODO: Se tendr que permitir indicar el signo de comparacin
				return " (CONVERT(varchar, " + entry.getKey() + ", 120)) " + entry.getValue().getOperador() + " '" + sdf.format(valor) + "'"; 
			}else if(valor instanceof Boolean){
				return ((Boolean)valor)? entry.getKey() + " " + entry.getValue().getOperador() + " 1" : 
					entry.getKey() + " " + entry.getValue().getOperador() + " 0";
			}
			// En otro caso devolvemos el String del campo
			return entry.getKey() + entry.getValue().getOperador() + String.valueOf(valor);
		}
		return "";
	}
	
	private static String construirCondicionIn(Object valor){
		String valorDevolver = "";
		String[] datosArray = ((String[])valor);
		int indice;
		for(indice = 1; indice <= datosArray.length; indice++){
			if(indice == 1){
				valorDevolver += "(";
			}
			if(indice != 1){
				valorDevolver += ",";
			}
			valorDevolver += datosArray[indice - 1];
			if(indice == datosArray.length){
				valorDevolver += ")";
			}
		}
		return valorDevolver;
	}
	
	public static String obtieneCriteriosOr(Object objeto, Map<String, String> listaParametros){
		// Criterios que se devuelve con el where formado
		String criterios = "";
		// Mapa de criterios para formar el where
		List<String> listaCriterios = new ArrayList<String>();
		// Itera sobre todos los metodos que se quieren poner en el WHERE
		for (Map.Entry<String, String> entry: listaParametros.entrySet()) {
			String nombreMetodo = entry.getKey();
			String nombreColumna = entry.getValue();

			try {
				// Obtiene por reflexion el metodo del que saldra el valor
				Method metodo = objeto.getClass().getMethod(nombreMetodo);
				try {
					// Invoca el metodo para obtener el valor
					Object valor = metodo.invoke(objeto, new Object[0]);
					if(valor != null){
						String criterio = obtieneValorCriterio(nombreColumna, valor);
						if(!criterio.trim().equals("")){
							listaCriterios.add(criterio);
						}
					}
				} catch (Exception e) {
					LOGGER.error("Se ha producido un error al invocar " + nombreMetodo, e);
				}
			} catch (NoSuchMethodException e) {
				LOGGER.error("No se ha encontrado el metodo " + nombreMetodo, e);
			}
		}

		// Creamos el iterador para ir formando el where
		Iterator<String> iteradorCriterios = listaCriterios.iterator();
		int contador = 0;
		// Para cada criterio
		while(iteradorCriterios.hasNext()){
			// parmetro que se aade
			String criterio = (String)iteradorCriterios.next();
			if(contador == 0){
				criterios += " WHERE " + criterio;
			}else{
				criterios += " OR " + criterio;
			}
			contador++;
		}
		// Devolvemos los criterios del where para la consulta
		return criterios;
	}
	
	/**
	 * Funcin que te crea un criterio de bsqueda para una query
	 * @param objeto Objeto del que se extraen los datos para el criterio
	 * @param listaParametros listado de los parmetros del objeto que se quieren controlar junto con la condicin
	 * 							que se quiere realizar
	 * @return Cadena con los criterios de la query
	 */
	public static String obtieneCriteriosConCondicionContinuarOr(Object objeto, Map<String, Map<String, TipoCondicion>> listaParametros){
		// Criterios que se devuelve con el where formado
		String criterios = "";
		// Mapa de criterios para formar el where
		List<String> listaCriterios = new ArrayList<String>();
		// Itera sobre todos los metodos que se quieren poner en el WHERE
		for (Map.Entry<String, Map<String, TipoCondicion>> entry: listaParametros.entrySet()) {
			String nombreMetodo = entry.getKey();
			Map<String, TipoCondicion> nombreColumna = entry.getValue();

			try {
				// Obtiene por reflexion el metodo del que saldra el valor
				Method metodo = objeto.getClass().getMethod(nombreMetodo);
				try {
					// Invoca el metodo para obtener el valor
					Object valor = metodo.invoke(objeto, new Object[0]);
					if(valor != null){
						String criterio = obtieneValorCriterioConCondicion(nombreColumna, valor);
						if(!criterio.trim().equals("")){
							listaCriterios.add(criterio);
						}
					}
				} catch (Exception e) {
					LOGGER.error("Se ha producido un error al invocar " + nombreMetodo, e);
				}
			} catch (NoSuchMethodException e) {
				LOGGER.error("No se ha encontrado el metodo " + nombreMetodo, e);
			}
		}

		// Creamos el iterador para ir formando el where
		Iterator<String> iteradorCriterios = listaCriterios.iterator();
		int contador = 0;
		// Para cada criterio
		while(iteradorCriterios.hasNext()){
			// parmetro que se aade
			String criterio = (String)iteradorCriterios.next();
			if(contador == 0){
				criterios += " AND " + criterio;
			}else{
				criterios += " OR " + criterio;
			}
			contador++;
		}
		// Devolvemos los criterios del where para la consulta
		return criterios;
	}
}
