package es.alten.diputacion.add.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

//import javax.xml.bind.JAXBContext;
//import javax.xml.bind.JAXBException;
//import javax.xml.bind.Unmarshaller;


/**
 * 
 * Utilidades utilies para la manipulacion de strings.
 * @author PGF
 */
public class StringUtils {

	/**
	 * Default buffer size: 16 Kb
	 */
	private static final int BUFFER_SIZE			= 16384;

	public static final String UTF8_ENCODING		= "UTF-8";
	public static final String ISO_8859_1_ENCODING	= "ISO-8859-1";
	protected static final char[] readBuffer = new char[512];

	/**
	 * 
	 */
	public static String parse(Reader reader) throws IOException {

		if(reader == null) {
			throw new IllegalArgumentException("reader cannot be null");
		}

		char[] buffer = new char[BUFFER_SIZE];
		StringBuilder stringBuilder = new StringBuilder();
		int length;

        while ((length = reader.read(buffer)) != -1){
            stringBuilder.append(buffer, 0, length);
        }
        return stringBuilder.toString();
	}

	/**
	 * 
	 */
	public static String parse(InputStream inputStream, String encoding) throws IOException {

		if(inputStream == null) {
			throw new IllegalArgumentException("inputStream cannot be null");
		}

		Reader inputStreamReader = new InputStreamReader(inputStream, encoding);
		return parse(inputStreamReader);
	}

	/**
	 * 
	 */
	public static String parse(InputStream inputStream) throws IOException {

		if(inputStream == null) {
			throw new IllegalArgumentException("inputStream cannot be null");
		}

		Reader inputStreamReader = new InputStreamReader(inputStream, UTF8_ENCODING);
		return parse(inputStreamReader);
	}
	/**
	 * 
	 */
	public synchronized String loadFromResource(String resource, String encoding) throws IOException  {
			InputStreamReader reader = new InputStreamReader( this.getClass().getResourceAsStream(resource), encoding );
			return loadFromResource(reader);
	}

	/**
	 * 
	 */
	public synchronized String loadFromResource(InputStream resource) throws IOException   {
		return loadFromResource(new InputStreamReader( resource ));
	}

	/**
	 * 
	 */
	public synchronized String loadFromResource(InputStreamReader reader) throws IOException  {
		StringBuilder builder = new StringBuilder();
		int readChars;
		while ( (readChars = reader.read(readBuffer, 0, readBuffer.length)) != -1 ){
			builder.append(readBuffer, 0, readChars);
		}
		return builder.toString();
	}

	/**
	 * 
	 */
//	public synchronized Object loadFromXML(String _nameSpace, String _resource) throws JAXBException, IOException  {
//
//        JAXBContext jc = null;
//        Object message = null;
//        Unmarshaller unmarshaller = null;
//
//		String xmlMessege = loadFromResource(_resource);
//		jc = JAXBContext.newInstance(_nameSpace);
//		unmarshaller = jc.createUnmarshaller();
//		message = (Object) unmarshaller.unmarshal(new StringReader(xmlMessege));
//
//
//        return message;
//	}

	/**
	 * 
	 */
	public synchronized String loadFromResource(String resource) throws IOException  {
		return loadFromResource(resource, UTF8_ENCODING);
	}

	/**
	 * 
	 */
	public static String convertToHex(byte[] data) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < data.length; i++) {
        	int halfbyte = (data[i] >>> 4) & 0x0F;
        	int two_halfs = 0;
        	do {
	        	if ((0 <= halfbyte) && (halfbyte <= 9))
	                buf.append((char) ('0' + halfbyte));
	            else
	            	buf.append((char) ('a' + (halfbyte - 10)));
	        	halfbyte = data[i] & 0x0F;
        	} while(two_halfs++ < 1);
        }
        return buf.toString();
    }

	/**
	 * 
	 */
	public static String MD5(String text) throws NoSuchAlgorithmException, UnsupportedEncodingException  {
		MessageDigest md;
		md = MessageDigest.getInstance("MD5");
		byte[] md5hash = new byte[32];
		md.update(text.getBytes("iso-8859-1"), 0, text.length());
		md5hash = md.digest();
		return convertToHex(md5hash);
	}

}